/* ----------------------------------
CREACIO TAULA LOG 
-----------------------------------*/
 
DROP TABLE BD_TFC.LOG CASCADE CONSTRAINTS; 
CREATE TABLE BD_TFC.LOG(
  idlog       Integer               CONSTRAINT PK_idlog      PRIMARY KEY,
  nompro      VARCHAR2(50 BYTE)     CONSTRAINT PK_nompro      NOT NULL,
  datahora    VARCHAR2(50 BYTE)     CONSTRAINT PK_datahora    NOT NULL,
  pinlog      VARCHAR2(500 BYTE)    ,
  poutlog     VARCHAR2(500 BYTE)    ,
  RSP         VARCHAR2(350 BYTE)    CONSTRAINT PK_RSP         NOT NULL
)
TABLESPACE TS_TFC;

-- CREACIO DEL AUTONUMERIC PER TAULA LOG


DROP SEQUENCE BD_TFC.S_LOGT;
CREATE SEQUENCE BD_TFC.S_LOGT INCREMENT BY 1 START WITH 1;


/* ----------------------------------
CREACIO TAULA Pais
-----------------------------------*/
DROP TABLE BD_TFC.PAIS CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.pais
  (
    idpais      integer       NOT NULL,
    capital     VARCHAR2(30)  NOT NULL,
    nompais     VARCHAR2(60)  NOT NULL,
    continent   VARCHAR2(20)  NOT NULL,
    situacio    VARCHAR2(60)  NOT NULL,
    nomoficial  VARCHAR2(60)  NOT NULL,
    ididioma    Integer       NOT NULL,
    nomidioma   VARCHAR2(20)  NOT NULL,
    CONSTRAINT  PK_idpais PRIMARY KEY (idpais)
  )
  TABLESPACE TS_TFC;
  
  
  /* ----------------------------------
CREACIO TAULA PARTIT
-----------------------------------*/
DROP TABLE BD_TFC.PARTIT CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.PARTIT
  (
    idpartit          integer       NOT NULL,
    idpaispartit      integer       NOT NULL,
    nompartit         VARCHAR2(60)  NOT NULL,
    idsigles          VARCHAR2(15)  NOT NULL,
    CONSTRAINT  PK_idpartit         PRIMARY KEY (idpartit),
    CONSTRAINT  FK_idpaispartit     FOREIGN KEY (idpaispartit) REFERENCES BD_TFC.PAIS (idpais)
    
  )
  TABLESPACE TS_TFC;
  
  /* ----------------------------------
CREACIO TAULA CARREC
-----------------------------------*/
DROP TABLE BD_TFC.CARREC CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.CARREC
  (
    idcarrec    integer       NOT NULL,
    nomcarrec   VARCHAR2(30)  NOT NULL,
    CONSTRAINT  PK_idcarrec   PRIMARY KEY (idcarrec)
   )
  TABLESPACE TS_TFC;
  
  /* ----------------------------------
CREACIO TAULA PERSONA
-----------------------------------*/
DROP TABLE BD_TFC.PERSONA CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.PERSONA
  (
    idpersona         Integer       NOT NULL,
    iddni             VARCHAR2(20)  NOT NULL,
    datanaixement     Date          NOT NULL,
    datadefuncio      Date                  ,
    idpaisnaixement   Integer       NOT NULL,
    nom               VARCHAR2(60)  NOT NULL,
    sexe              VARCHAR2(02)  NOT NULL,
    estatcivil        VARCHAR2(02)          ,
    nivellestudis     VARCHAR2(02)          ,
    idpaisresidencia  Integer       NOT NULL,
    adrea            VARCHAR2(60)  NOT NULL,
    telefon           VARCHAR2(20)          ,
    mail              VARCHAR2(60)          ,
    CONSTRAINT  PK_idpersona           PRIMARY KEY (idpersona),
    CONSTRAINT  FK_idpaisnaixement     FOREIGN KEY (idpaisnaixement) REFERENCES BD_TFC.PAIS (idpais),
    CONSTRAINT  FK_idpaisresidencia    FOREIGN KEY (idpaisresidencia) REFERENCES BD_TFC.PAIS (idpais)
    
  )
  TABLESPACE TS_TFC;
  
/* ----------------------------------
CREACIO TAULA PARLAMENT
-----------------------------------*/
DROP TABLE BD_TFC.PARLAMENT CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.PARLAMENT
  (
    idparlament       Integer       NOT NULL,
    tipusp            VARCHAR2(30)  NOT NULL,
    nom               VARCHAR2(60)  NOT NULL,
    totalp            Integer       NOT NULL,
    adrea            VARCHAR2(60)  NOT NULL,
    telefon           VARCHAR2(20)  NOT NULL,
    fax               VARCHAR2(20)  NOT NULL,  
    web               VARCHAR2(30)  NOT NULL, 
    mail              VARCHAR2(30)  NOT NULL,
    datacreacio       Date          NOT NULL,
    idpaisparlament   Integer       NOT NULL,
    inicilegislatura  Date          NOT NULL,
    finallegislatura  Date          NOT NULL,
    idbanc            VARCHAR2(60)  NOT NULL,
    actiu             VARCHAR2(01)  NOT NULL,
    
    CONSTRAINT  PK_idparlament      PRIMARY KEY (idparlament),
    CONSTRAINT  FK_idpaisparlament  FOREIGN KEY (idpaisparlament)           REFERENCES BD_TFC.PAIS(idpais)
  )
    
  TABLESPACE TS_TFC;

-- CREACIO DEL CAMP AUTONUMERIC PER LA  TAULA PARLAMENT 

DROP SEQUENCE BD_TFC.S_PARLAMENT;

CREATE SEQUENCE BD_TFC.S_PARLAMENT INCREMENT BY 1 START WITH 1;




  /* ----------------------------------
CREACIO TAULA PARLAMENTARI
-----------------------------------*/
DROP TABLE BD_TFC.PARLAMENTARI CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.PARLAMENTARI
  (
    idparlamentari    Integer       NOT NULL,
    idpersona         Integer       NOT NULL,
    dataalta          Date          NOT NULL,
    adrea            VARCHAR2(60)  NOT NULL,
    telefon           VARCHAR2(20)  NOT NULL,
    fax               VARCHAR2(20)  NOT NULL,  
    mail              VARCHAR2(30)  NOT NULL,
    idvisa            VARCHAR2(60)  NOT NULL,
    idpartit          Integer       NOT NULL,
    idcarrec          Integer       NOT NULL,
    idparlament       Integer       NOT NULL,
    actiu             VARCHAR2(01)  NOT NULL,
    
    CONSTRAINT  PK_idperlamentari   PRIMARY KEY (idparlamentari),
    CONSTRAINT  FK_idpersona        FOREIGN KEY (idpersona)        REFERENCES BD_TFC.PERSONA  (idpersona),
    CONSTRAINT  FK_idpartit         FOREIGN KEY (idpartit)         REFERENCES BD_TFC.PARTIT   (idpartit),
    CONSTRAINT  FK_idcarrec         FOREIGN KEY (idcarrec)         REFERENCES BD_TFC.CARREC   (idcarrec),
    CONSTRAINT  FK_idparlament      FOREIGN KEY (idparlament)      REFERENCES BD_TFC.PARLAMENT(idparlament),
    CONSTRAINT  FK_idcarrec         FOREIGN KEY (idcarrec)         REFERENCES BD_TFC.CARREC   (idcarrec)

  )
  TABLESPACE TS_TFC;
  
-- CREACIO DEL CAMP AUTONUMERIC PER LA TAULA PARLAMENTARI 
  
DROP SEQUENCE BD_TFC.S_PARLAMENTARI;

CREATE SEQUENCE BD_TFC.S_PARLAMENTARI INCREMENT BY 1 START WITH 1;

 
/* ----------------------------------
CREACIO TAULA ESTAT
-----------------------------------*/
DROP TABLE BD_TFC.ESTAT CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.ESTAT
  (
    idestat          integer       NOT NULL,
    nomestat         VARCHAR2(30)  NOT NULL,
    CONSTRAINT  PK_idestat         PRIMARY KEY (idestat)
  )
  TABLESPACE TS_TFC;
   
  /* ----------------------------------
CREACIO TAULA TIPUS_DESPESA
-----------------------------------*/
DROP TABLE BD_TFC.TIPUS_DESPESA  CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.TIPUS_DESPESA
  (
    idtipus          integer       NOT NULL,
    nomtipus         VARCHAR2(80)  NOT NULL,
    CONSTRAINT  PK_idtipus         PRIMARY KEY (idtipus)
  )
  TABLESPACE TS_TFC;
  
  /* ----------------------------------
CREACIO TAULA PROVEIDOR
-----------------------------------*/
DROP TABLE BD_TFC.PROVEIDOR CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.PROVEIDOR
  (
    idproveidor       Integer       NOT NULL,
    idnif             VARCHAR2(20)  NOT NULL,
    nomcomercial      VARCHAR2(60)  NOT NULL,
    idactivitat       VARCHAR2(60)  NOT NULL,
    idpaisproveidor   Integer       NOT NULL,
    adrea            VARCHAR2(60)  NOT NULL,
    telefon           VARCHAR2(20)  NOT NULL,
    fax               VARCHAR2(20)  NOT NULL,  
    mail              VARCHAR2(60)  NOT NULL,
    web               VARCHAR2(60)  NOT NULL, 
    
    CONSTRAINT  PK_idproveidor      PRIMARY KEY (idproveidor),
    CONSTRAINT  FK_idpaisproveidor  FOREIGN KEY (idpaisproveidor)      REFERENCES BD_TFC.PAIS(idpais)
  )
  TABLESPACE TS_TFC;
  
  
  /* ----------------------------------
CREACIO TAULA  DESPESA
-----------------------------------*/
DROP TABLE BD_TFC.DESPESA CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.DESPESA
  (
    iddespesa         Integer       NOT NULL,
    idcategoria       Integer       NOT NULL,
    datadespesa       Date          NOT NULL,
    descripcio        VARCHAR2(60)  NOT NULL,
    justificacio      VARCHAR2(60)  NOT NULL,
    urlimatge         VARCHAR2(60)  NOT NULL,
    dataestat         Date          NOT NULL,
    idpagament        VARCHAR2(60)  NOT NULL,
    importbrut        Number        NOT NULL,
    tx100iva          Number        NOT NULL,
    importtotal       Number        NOT NULL,
    idproveidor       Integer       NOT NULL,
    idestat           Integer       NOT NULL,
    idparlamentd      Integer       NOT NULL,
    idparlamentarid   Integer       ,
    idtipus           Integer       NOT NULL,
    idpartitd         Integer       ,
    actiu             VARCHAR2(1)   NOT NULL,
    
    CONSTRAINT  PK_iddespesa        PRIMARY KEY (iddespesa),
    CONSTRAINT  FK_idproveidor      FOREIGN KEY (idproveidor)      REFERENCES BD_TFC.PROVEIDOR    (idproveidor),
    CONSTRAINT  FK_idestat          FOREIGN KEY (idestat)          REFERENCES BD_TFC.ESTAT        (idestat),
    CONSTRAINT  FK_idparlamentd     FOREIGN KEY (idparlamentd)     REFERENCES BD_TFC.PARLAMENT    (idparlament),
    CONSTRAINT  FK_idtipus          FOREIGN KEY (idtipus)          REFERENCES BD_TFC.TIPUS_DESPESA(idtipus)
   )
  TABLESPACE TS_TFC;

-- CREACIO DEL CAMP AUTONUMERIC PER LA TAULA DESPESA

DROP SEQUENCE BD_TFC.S_DESPESA;

CREATE SEQUENCE BD_TFC.S_DESPESA INCREMENT BY 1 START WITH 1;

  
  /* ----------------------------------
CREACIO TAULA E1
-----------------------------------*/
DROP TABLE BD_TFC.E1 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E1
  (
    idparlament    integer       NOT NULL,
    resultat       NUMBER        NOT NULL,
    CONSTRAINT  PK_E1       PRIMARY KEY (idparlament),
    CONSTRAINT  FK_E1       FOREIGN KEY (idparlament)   
    REFERENCES BD_TFC.PARLAMENT(idparlament)
  )
  TABLESPACE TS_TFC;
  
  /* ----------------------------------
CREACIO TAULA E2
-----------------------------------*/
DROP TABLE BD_TFC.E2 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E2
  (
     idany          VARCHAR2(4)   NOT NULL,
     idparlament    integer       NOT NULL,
     resultat       NUMBER        NOT NULL,
    CONSTRAINT  PK_E2                    PRIMARY KEY (idany,  idparlament),
    CONSTRAINT  FK_E2                    FOREIGN KEY (idparlament) 
    REFERENCES BD_TFC.PARLAMENT(idparlament)
  )
  TABLESPACE TS_TFC;
  
  /* ----------------------------------
CREACIO TAULA E3
-----------------------------------*/
DROP TABLE BD_TFC.E3 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E3
  (
    idany          VARCHAR2(4)   NOT NULL,
    resultat       INTEGER       NOT NULL,
    CONSTRAINT  PK_E3             PRIMARY KEY (idany)
   
  )
  TABLESPACE TS_TFC;
  
  
  /* ----------------------------------
CREACIO TAULA E4
-----------------------------------*/
DROP TABLE BD_TFC.E4 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E4
  (
    idparlamentari            Integer       NOT NULL,
    resultat                  Number        NOT NULL,
    CONSTRAINT  PK_E4         PRIMARY KEY (idparlamentari),
    CONSTRAINT  FK_E4         FOREIGN KEY (idparlamentari) 
    REFERENCES BD_TFC.PARLAMENTARI(idparlamentari)
  )
  TABLESPACE TS_TFC;
  
  /* ----------------------------------
CREACIO TAULA E5
-----------------------------------*/
DROP TABLE BD_TFC.E5 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E5
  (  
    idany            VARCHAR2(4)   NOT NULL,
    resultat         Number        NOT NULL,
    CONSTRAINT  PK_E5         PRIMARY KEY (idany, idparlamentari),
    CONSTRAINT  FK_E5         FOREIGN KEY (idparlamentari) 
    REFERENCES BD_TFC.PARLAMENTARI(idparlamentari)
  )
  TABLESPACE TS_TFC;
  
  /* ----------------------------------
CREACIO TAULA E6
-----------------------------------*/
DROP TABLE BD_TFC.E6 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E6
  (
    idany          VARCHAR2(4)   NOT NULL,
    idparlamentari0           Integer       NOT NULL,
    idparlamentari9           Integer       NOT NULL,
     CONSTRAINT  PK_E6         PRIMARY KEY (idany),
    CONSTRAINT  FK_E6MIN         FOREIGN KEY (idparlamentari0)      REFERENCES BD_TFC.PARLAMENTARI(idparlamentari),
    CONSTRAINT  FK_E6MAX         FOREIGN KEY (idparlamentari9)      REFERENCES BD_TFC.PARLAMENTARI(idparlamentari)
  )
  TABLESPACE TS_TFC;
  
  
    /* ----------------------------------
CREACIO TAULA E7
-----------------------------------*/
DROP TABLE BD_TFC.E7 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E7 
  (
    idany          VARCHAR2(4)   NOT NULL,
    nom           VARCHAR(60)   NOT NULL,
    resultat       NUMBER        NOT NULL,
    CONSTRAINT  PK_E7         PRIMARY KEY (idany)
      )
  TABLESPACE TS_TFC;
  
  
  /* ----------------------------------
CREACIO TAULA E8
-----------------------------------*/
DROP TABLE BD_TFC.E8 CASCADE CONSTRAINTS;
CREATE TABLE BD_TFC.E8 
  (
    idany          VARCHAR2(4)   NOT NULL,
   resultat       NUMBER        NOT NULL,
    CONSTRAINT  PK_E8         PRIMARY KEY (idany)
      )
  TABLESPACE TS_TFC;
  ------------------------------------------------------------
  
  commit;
  
  