CREATE OR REPLACE PACKAGE                                                                                                           PAQ_CONSULTA AS
/******************************************************************************
   NAME:       CONSULTA_A
  *            Donat un parlament, un any i un parlamentari:
  *            Obtenir  el llistat de totes les despeses associades al parlamentari 
  *            en aquell any, ordenat de forma descendent pel valor de cada despesa.
  *            PROCESSAR SOLS ELS PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
******************************************************************************/
PROCEDURE CONSULTA_A (P_ANY IN VARCHAR2,  P_IDPARLAMENTARI IN INTEGER,  P_IDPARLAMENT IN INTEGER,  RSP  OUT  VARCHAR2); 


/******************************************************************************
   NAME:       CONSULTA_B
  *            Donat un parlament, un any i un parlamentari:
  *            Obtenir  el llistat de totes les despeses associades al parlamentari 
  *            en aquell any, ordenat de forma descendent pel valor de cada despesa.
  *            PROCESSAR SOLS ELS PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
******************************************************************************/
PROCEDURE CONSULTA_B (P_ANY IN VARCHAR2,  P_IDPARLAMENT IN INTEGER,  RSP  OUT  VARCHAR2); 


/******************************************************************************
   NAME:       CONSULTA_C
  *            Donat un any: 
  *           ?	Nom del parlament
  *           ?	Nombre de parlamentaris 
  *           ?	Despesa mitja dels seus parlamentaris (tenint en compte noms les despeses directes realitzades pels parlamentaris).
  *           ?	Suma de les despeses totals associades als parlamentaris.
  *           ?	Suma de les despeses totals associades al parlament (s a dir les que no estan associades a un parlamentari concret).
  *           ?	Suma de totes les despeses. 

  *            PROCESSAR SOLS ELS PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
******************************************************************************/
PROCEDURE CONSULTA_C (P_ANY IN VARCHAR2, RSP OUT VARCHAR2);
/******************************************************************************
   NAME:       CONSULTA_D
  *            Donat un parlament, un any i un parlamentari:
  *            Obtenir el llistat de la despesa segmentada per categories de despesa,
               s a dir, un llistat que retorni les columnes segents:
                ?	Codi de la categoria de despesa.
                ?	Descripci de la categoria de despesa.
                ?	Suma de les despeses totals associades a la categoria de despesa, 
                  durant lany indicat i en el parlament indicat.
  
  *            PROCESSAR SOLS ELS PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
******************************************************************************/
PROCEDURE CONSULTA_D (P_ANY IN VARCHAR2,  P_IDPARLAMENT IN INTEGER, RSP   OUT VARCHAR2);


/******************************************************************************
   NAME:       CONSULTA_E
  *            Donat un parlament i un any: 
  *            Obtenir  el llistat el llistat de parlamentaris que superen la despesa
               mitja dels tots els parlamentaris daquell parlament durant lany indicat,
               incloent: 
                ?	El seu nom i cognom 
                ?	Partit poltic actual 
                ?	Suma de totes les despeses directes associades al parlamentari durant aquell any
                ?	Percentatge en que es supera la despesa mitjana  

  *            PROCESSAR SOLS ELS PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
******************************************************************************/
PROCEDURE CONSULTA_E (P_ANY IN VARCHAR2,  P_IDPARLAMENT IN INTEGER, RSP   OUT VARCHAR2);


/******************************************************************************
   NAME:       CONSULTA_F
  *            f.	Donat un any: 
  *            Obtenir  el llistat  el llistat de tots els parlaments amb el seu estat 
               comptable, incloent:
                ?	Nom del parlament 
                ?	Nombre de parlamentaris
                ?	Suma de totes les despeses pendents daprovaci.
                ?	Suma de totes les despeses aprovades.
                ?	Suma de totes les despeses pendents dabonament.
                ?	Suma de totes les despeses abonades.

  *            PROCESSAR SOLS ELS PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
******************************************************************************/
PROCEDURE CONSULTA_F (P_ANY IN VARCHAR2, RSP OUT VARCHAR2);
 
 
END PAQ_CONSULTA;
/


CREATE OR REPLACE PACKAGE BODY                                                                                                           PAQ_CONSULTA AS
   /* **********************************************************************************************************
  *  NOM:         CONSULTA_A
  *  DESCRIPCIO:
  *            Donat un parlament, un any i un parlamentari:
  *            Obtenir  el llistat de totes les despeses associades al parlamentari 
  *            en aquell any, ordenat de forma descendent pel valor de cada despesa.
  *            PROCESSAR SOLS ELS PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
  *  PARAMETERS:
  *  IN PUT ->  P_ANY,  P_IDPARLAMENTARI, P_IDPARLAMENT ,
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/

   PROCEDURE CONSULTA_A (P_ANY              IN     VARCHAR2
                        ,P_IDPARLAMENTARI   IN     INTEGER
                        ,P_IDPARLAMENT      IN     INTEGER
                        ,RSP                   OUT VARCHAR2) IS
      r_idparlament      INTEGER;
      r_idparlamentari   INTEGER;
      r_any              VARCHAR2 (4);

      w_actiu            VARCHAR2 (1);
      p_nompro           VARCHAR2 (50);
      p_inlog            VARCHAR2 (500);
      p_outlog           VARCHAR2 (500);
      p_rsp              VARCHAR2 (350);
      err_msg            VARCHAR2 (256);
      err_num            NUMBER;
      V_NUM              NUMBER;

      wnulls             EXCEPTION;
      er_iddupli         EXCEPTION;
      er_dates           EXCEPTION;
      es_histo           EXCEPTION;
      notfnd1            EXCEPTION;
      notfnd2            EXCEPTION;
      es_histo1          EXCEPTION;
      es_histo2          EXCEPTION;

      /*  TAULES:
      DESPESA = D.
      PARLAMENTARI= P.
       */
      CURSOR C_CNS_A IS
         SELECT   D.*
         FROM     DESPESA D, PARLAMENTARI P
         WHERE    D.IDPARLAMENTARID = P.IDPARLAMENTARI
         AND      P.ACTIU = 'S'
         AND      TO_CHAR (D.DATADESPESA, 'YYYY') = r_any
         AND      D.IDPARLAMENTARID = r_idparlamentari
         AND      D.IDPARLAMENTD = r_idparlament
         ORDER BY IMPORTTOTAL DESC;
   BEGIN
      -- preparar les dades per LOG

      r_any := p_any;
      r_idparlament := p_idparlament;
      r_idparlamentari := p_idparlamentari;
      p_nompro := 'Consulta_a';
      p_inlog :=
         r_any || ',    ' || p_idparlament || ',    ' || p_idparlamentari;
      p_outlog := 'OK';
      RSP := 'OK';

      --Controlar que els camps no siguin nuls
      IF (r_any IS NULL)
         OR (r_idparlamentari IS NULL)
         OR (r_idparlament IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- validar les dades d'entrada ----
      if PAQ_UTI.ANYOK(r_any) <> 'OK' then
         RAISE er_dates;
      END IF;

-- VALIDAR QUE EL PARLAMENT ESTA ACTIU
      w_actiu := PAQ_PARLAMENT.estatParlament (r_idparlament);

      IF w_actiu = 'N' THEN
         RAISE notfnd1;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo1;
      END IF;
-- VALIDAR QUE EL PARLAMENTARI ESTA ACTIU
      w_actiu := PAQ_PARLAMENTARI.estatParlamentari (r_idparlamentari);

      IF w_actiu = 'N' THEN
         RAISE notfnd2;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo2;
      END IF;

      ---   Escriu la capalera del lllistat  ---
      DBMS_OUTPUT.PUT_LINE (
            'CONSULTA A: DESPESES DE L''ANY: '
         || p_any
         || ' DEL PARLAMENT: '
         || P_IDPARLAMENT
         || ' I DEL PARLAMENTARI: '
         || P_IDPARLAMENTARI);

      ---   executa el llistat de la select declarada  a la definici del cursor ---
      FOR V_CNS IN C_CNS_A LOOP
        V_NUM := V_CNS.IMPORTTOTAL;
         DBMS_OUTPUT.PUT_LINE (
            (trunc(v_num,2)) ||' | ' || V_CNS.DESCRIPCIO);
            
      END LOOP;
      
      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: Consulta A.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Consulta A erronia. El Parlament no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo1 THEN
         rsp := 'ERROR:  Consulta  A erronia. El Parlament  s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd2 THEN
         rsp := 'ERROR: Consulta A erronia. El Parlamentari no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo2 THEN
         rsp := 'ERROR: Consulta A  erronia. El Parlamentari s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Consulta_ A. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
          p_nompro:= 'Consulta_a';
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: Consulta_A[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END CONSULTA_A;

   /*  *********************************************************************************************************
  *  NOM:         CONSULTA_B
  *  DESCRIPCIO:
  *    DONAT UN PARLAMENT i UN ANY :
  *    OBTENIR EL LLISTAT DELS PARLAMENTARIS AMB LA DESPESA DIRECTA QUE HA FET CADASCUN, INCLOENT:
  *        EL SEU NOM I COGNOMS
  *        PARTIT POLTIC ACTUAL
  *        SUMA DE TOTES LES SEVES DESPESES DIRECTES ASSOCIADES
  *        ORDENAT DE FORMA DESCENDENT PEL VALOR DE LA SUMA DE LES DESPESES.
  *  PARAMETERS:
  *  IN PUT ->
  *              P_ANY,   P_IDPARLAMENT ,
   *  OUTPUT ->
  *             RSP ;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/

   PROCEDURE CONSULTA_B (P_ANY           IN     VARCHAR2
                        ,P_IDPARLAMENT   IN     INTEGER
                        ,RSP                OUT VARCHAR2) IS
      r_idparlament      INTEGER;
      r_idparlamentari   INTEGER;
      r_any              VARCHAR2 (4);

      w_actiu            VARCHAR2 (1);
      p_nompro           VARCHAR2 (50);
      p_inlog            VARCHAR2 (500);
      p_outlog           VARCHAR2 (500);
      p_rsp              VARCHAR2 (350);
      err_msg            VARCHAR2 (256);
      err_num            NUMBER;
      V_NUM              NUMBER;

      wnulls             EXCEPTION;
      er_iddupli         EXCEPTION;
      er_dates           EXCEPTION;
      notfnd1            EXCEPTION;
      es_histo1          EXCEPTION;


      /*  TAULES:  DESPESA = D.; PARLAMENTARI= PA.; PARTIT = PI.
                   PREPARAR EL CURSOR AMB LA SELECT  */

      CURSOR C_CNS_B IS
         SELECT   PA.IDPARLAMENTARI
                 ,PE.NOM
                 ,PI.NOMPARTIT
                 ,NVL (SUM (D.IMPORTTOTAL), 0) AS IMPORT
         FROM     PARLAMENTARI PA
                 ,DESPESA D
                 ,PERSONA PE
                 ,PARTIT PI
         WHERE    PA.IDPARLAMENT = r_idparlament
         AND      D.IDPARLAMENTARID(+) = PA.IDPARLAMENTARI
         AND      PA.IDPERSONA = PE.IDPERSONA
         AND      PA.IDPARTIT = PI.IDPARTIT
         GROUP BY PA.IDPARLAMENTARI, PE.NOM, PI.NOMPARTIT
         ORDER BY IMPORT DESC;
   BEGIN
      -- preparar les dades per LOG
      r_any := p_any;
      r_idparlament := p_idparlament;
      p_nompro := 'Consulta_B';
      p_inlog := r_any || ',    ' || p_idparlament;

      p_outlog := 'OK';
      RSP := 'OK';

      --Controlar que els camps no siguin nuls
      IF (r_any IS NULL) OR (r_idparlament IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- validar les dades d'entrada ----
      IF r_any > 2000 AND r_any <= 2013 THEN
         NULL;
      ELSE
         RAISE er_dates;
      END IF;
    -- VALIDAR QUE EL PARLAMENT EXISTEIX I ESTA ACTIU
      w_actiu := PAQ_PARLAMENT.estatParlament (r_idparlament);

      IF w_actiu IS NULL THEN
         RAISE notfnd1;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo1;
      END IF;

      -- ESCRIU LA CAPALERA DEL LLISTAT
      DBMS_OUTPUT.PUT_LINE (
            'CONSULTA B: DESPESES  DELS PARLAMENTARIS  L''ANY: '
         || p_any
         || ' DEL PARLAMENT: ' || p_idparlament);
         
         DBMS_OUTPUT.PUT_LINE (
            'NOM PARLAMENTARI  | NOM PARTIT | IMPORT ');
             DBMS_OUTPUT.PUT_LINE (
            '----------------------------------------');
        

      ---   executa el llistat de la select declarada  a la definici del cursor ---

      FOR V_CNS IN C_CNS_B LOOP
         V_NUM :=V_CNS.IMPORT;
         DBMS_OUTPUT.PUT_LINE (
         V_CNS.nom|| ' | ' || V_CNS.NOMPARTIT || ' | '  || (trunc(v_num,2)));
            

      END LOOP;
      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: Consuta_B.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Consulta B erronia. El Parlament no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo1 THEN
         rsp := 'ERROR:  Consulta B  erronia. El Parlament  s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Consulta B. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
          p_nompro:= 'Consulta_b';
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: Consulta_B[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END CONSULTA_B;

   /* *********************************************************************************************************
   *  NOM:         CONSULTA_C
   *  DESCRIPCIO:
   *     DONAT UN ANY,
   *     OBTENIR EL LLISTAT DE LA DESPESA TOTAL DEL TOTS ELS PARLAMENTS, INCLOENT:
   *         NOM DEL PARLAMENT
   *         NOMBRE DE PARLAMENTARIS
   *         DESPESA MITJA DELS SEUS PARLAMENTARIS (TENINT EN COMPTE NOMS LES DESPESES DIRECTES REALITZADES PELS PARLAMENTARIS).
   *         SUMA DE LES DESPESES TOTALS ASSOCIADES ALS PARLAMENTARIS.
   *         SUMA DE LES DESPESES TOTALS ASSOCIADES AL PARLAMENT (S A DIR LES QUE NO ESTAN ASSOCIADES A UN PARLAMENTARI CONCRET).
   *         SUMA DE TOTES LES DESPESES.

   *        PROCESSAR SOLS LES DESPESES,  PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
   *  PARAMETERS:
   *  IN PUT ->
   *              P_ANY,
    *  OUTPUT ->
   *             RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***************************************************************************************************************/

   PROCEDURE CONSULTA_C (P_ANY IN VARCHAR2, RSP OUT VARCHAR2) IS
      r_any      VARCHAR2 (4);
      w_actiu    VARCHAR2 (1);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      p_rsp      VARCHAR2 (350);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
      V_NUM      NUMBER;
      V_NUM2      NUMBER;

      wnulls     EXCEPTION;
      er_dates   EXCEPTION;


      /*  TAULES:  DESPESA = D. ; D1. ; D2.
                   PARLAMENT = PT.
          PREPARAR EL CURSOR AMB LA SELECT 
      */

      CURSOR C_CNS_C IS
         SELECT   NOM
                 ,TOTALP
                 ,DESPESA_PARLAMENTARIS
                 ,DESPESA_PARLAMENT
                 , (DESPESA_PARLAMENTARIS + DESPESA_PARLAMENT) AS DESPESA_TOTAL
                 ,DECODE(TOTALP,0,0,(DESPESA_PARLAMENTARIS / TOTALP)) AS MITJA_PARLAMENTARIS
         FROM     (SELECT PT.NOM
                         ,PT.TOTALP
                         ,NVL (
                             (SELECT SUM (D1.IMPORTTOTAL)
                              FROM   DESPESA D1
                              WHERE  D1.IDPARLAMENTD = PT.IDPARLAMENT
                              AND    D1.ACTIU = 'S'
                              AND    TO_CHAR (D1.DATADESPESA, 'YYYY') = P_ANY
                              AND    D1.IDCATEGORIA = 1)
                            ,0)
                             AS DESPESA_PARLAMENTARIS
                         ,NVL (
                             (SELECT SUM (D2.IMPORTTOTAL)
                              FROM   DESPESA D2
                              WHERE  D2.IDPARLAMENTD = PT.IDPARLAMENT
                              AND    D2.ACTIU = 'S'
                              AND    TO_CHAR (D2.DATADESPESA, 'YYYY') = P_ANY
                              AND    D2.IDCATEGORIA = 0)
                            ,0)
                             AS DESPESA_PARLAMENT
                   FROM   PARLAMENT PT, DESPESA D
                   WHERE  D.IDPARLAMENTD(+) = PT.IDPAISPARLAMENT
                   AND    NVL (D.ACTIU, 'S') = 'S'
                   AND    NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), P_ANY) = P_ANY)
         GROUP BY NOM
                 ,TOTALP
                 ,DESPESA_PARLAMENTARIS
                 ,DESPESA_PARLAMENT;
   BEGIN
      -- preparar les dades per LOG
      r_any := p_any;
      p_nompro := 'Consulta_C';
      p_inlog := r_any;

      p_outlog := 'OK';
      RSP := 'OK';

      --Controlar que els camps no siguin nuls
      IF (r_any IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- validar les dades d'entrada ----

      IF r_any > 2000 AND r_any <= 2013 THEN
         NULL;
      ELSE
         RAISE er_dates;
      END IF;

      -- ESCRIUT LA CAALERA DEL LLISTAT

      DBMS_OUTPUT.PUT_LINE (
            'CONSULTA C: DESPESES  DELS PARLAMENTARIS  L''ANY: '
         || p_any);
     DBMS_OUTPUT.PUT_LINE (
      'NOM PARLAMENT | NUMERO PARLAMENTARIS | DESPESA MITJA PARALMENTARIS | DESPESA TOTAL PARLAMENTARIS |'
      || ' DESPESA TOTAL PARLAMENT | TOTAL DESPESES'
  );
   DBMS_OUTPUT.PUT_LINE (
      '-------------------------------------------------------------------------------------------------'
      || '-----------------------------------------'
  );
      ---   executa el llistat de la select declarada  a la definici del cursor ---

      FOR V_CNS IN C_CNS_C LOOP
      V_NUM := V_CNS.despesa_total;
      V_NUM2:= V_CNS.mitja_parlamentaris; 
          DBMS_OUTPUT.PUT_LINE (
               V_CNS.nom
            || ' | '
            || V_CNS.totalp
            || ' | '
            || (trunc(v_num2,2))
            ||' | '
            || V_CNS.despesa_parlamentaris
            || ' | '
            || V_CNS.despesa_parlament
            || ' | '
            || (trunc(v_num,2))
            );
      END LOOP;
      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: Consuta_C.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Consulta C. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
          p_nompro:= 'Consulta_c';
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: Consulta_C[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END CONSULTA_C;

   /* ********************************************************************************************************
   *  NOM:         CONSULTA_D
   *  DESCRIPCIO:
   *     DONAT UN ANY, I UN PARLAMENT:
   *        OBTENIR  EL LLISTAT DE LA DESPESA SEGMENTADA PER CATEGORIES DE DESPESA AMB DETALL DE:
   *        CODI DE LA CATEGORIA DE DESPESA.
   *        DESCRIPCI DE LA CATEGORIA DE DESPESA.
   *        SUMA DE LES DESPESES TOTALS ASSOCIADES A LA CATEGORIA DE DESPESA, DURANT LANY INDICAT I EN EL PARLAMENT INDICAT.
   *        PROCESSAR SOLS LES DESPESES,  PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
   *  PARAMETERS:
   *  IN PUT -> P_ANY,
   *  OUTPUT -> RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***************************************************************************************************************/
   PROCEDURE CONSULTA_D (P_ANY           IN     VARCHAR2
                        ,P_IDPARLAMENT   IN     INTEGER
                        ,RSP                OUT VARCHAR2) AS
      r_any           VARCHAR2 (4);
      r_idparlament   INTEGER;
      w_actiu         VARCHAR2 (1);
      p_nompro        VARCHAR2 (50);
      p_inlog         VARCHAR2 (500);
      p_outlog        VARCHAR2 (500);
      p_rsp           VARCHAR2 (350);
      err_msg         VARCHAR2 (256);
      err_num         NUMBER;
      V_NUM           NUMBER;

      wnulls          EXCEPTION;
      er_dates        EXCEPTION;
      es_histo        EXCEPTION;
      notfnd1         EXCEPTION;
      es_histo1       EXCEPTION;


      /*  TAULES:   DESPESA = D. ; TIPUS_DESPESA  = TD.  
           PREPARAR EL CURSOR AMB LA SELECT   */
      CURSOR C_CNS_D IS
         SELECT   TD.IDTIPUS, TD.NOMTIPUS, SUM (D.IMPORTTOTAL) AS IMPORT
         FROM     despesa d, TIPUS_DESPESA TD
         WHERE    TO_CHAR (D.DATADESPESA, 'YYYY') = p_any
         AND      D.IDPARLAMENTD = p_idparlament
         AND      d.actiu = 'S'
         AND      D.IDTIPUS = TD.IDTIPUS
         GROUP BY TD.IDTIPUS, TD.NOMTIPUS;
   BEGIN
      r_any := p_any;
      r_idparlament := p_idparlament;
    -- preparar les dades per la taula LOG
      p_nompro := 'Consulta_d';
      p_inlog :=
         r_any || ',    ' || p_idparlament ;
      p_outlog := 'OK';
      RSP := 'OK';

      --Controlar que els camps no siguin nuls
      IF (r_any IS NULL) OR (r_idparlament IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- validar les dades d'entrada ----
      IF r_any > 2000 AND r_any <= 2013 THEN
         NULL;
      ELSE
         RAISE er_dates;
      END IF;
      -- validar que el parlament existeis i esta actiu
      w_actiu := PAQ_PARLAMENT.estatParlament (r_idparlament);

      IF w_actiu IS NULL THEN
         RAISE notfnd1;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo1;
      END IF;

      -- ESCRIUT LA CAALERA DEL LLISTAT

      DBMS_OUTPUT.PUT_LINE (
            'CONSULTA D: DESPESES  PER CATEGORIA DE DESPESA  DEL PARLAMENT'
         || p_idparlament
         || '    L''ANY: '
         || p_any);
      DBMS_OUTPUT.PUT_LINE (
            'TIPUS DE DESPESA  | DESCRIPCIO  | TOTAL IMPORT'
        );
       DBMS_OUTPUT.PUT_LINE (
            '----------------------------------------------'
        );
      ---   executa el llistat de la select declarada  a la definici del cursor ---

      FOR V_CNS IN C_CNS_D LOOP
      V_NUM := V_CNS.IMPORT;
         DBMS_OUTPUT.PUT_LINE (
            V_CNS.IDTIPUS || ' | ' || V_CNS.NOMTIPUS ||' | ' || (trunc(v_num,2)));
      END LOOP;
      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: Consuta_D['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Consulta D erronia. El Parlament no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo1 THEN
         rsp := 'ERROR:  Consulta D  erronia. El Parlament  s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Consulta D. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
          p_nompro:= 'Consulta_d';
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: Consulta_D[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END CONSULTA_D;

   /* ********************************************************************************************************
   *  NOM:         CONSULTA_E
   *  DESCRIPCIO:
   *       DONAT UN ANY, I UN PARLAMENT:
   *       EL LLISTAT DE PARLAMENTARIS QUE SUPEREN LA DESPESA MITJA DELS TOTS ELS PARLAMENTARIS DAQUELL PARLAMENT DURANT LANY INDICAT, INCLOENT:
   *       EL SEU NOM I COGNOM
   *       PARTIT POLTIC ACTUAL
   *       SUMA DE TOTES LES DESPESES DIRECTES ASSOCIADES AL PARLAMENTARI DURANT AQUELL ANY
   *       PERCENTATGE EN QUE ES SUPERA LA DESPESA MITJANA
   *       PROCESSAR SOLS LES DESPESES,  PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
   *  PARAMETERS:
   *  IN PUT -> P_ANY,
   *  OUTPUT -> RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***************************************************************************************************************/

   PROCEDURE CONSULTA_E (P_ANY           IN     VARCHAR2
                        ,P_IDPARLAMENT   IN     INTEGER
                        ,RSP                OUT VARCHAR2) IS
      r_idparlament   INTEGER;
      r_any           VARCHAR2 (4);

      w_actiu         VARCHAR2 (1);
      p_nompro        VARCHAR2 (50);
      p_inlog         VARCHAR2 (500);
      p_outlog        VARCHAR2 (500);
      p_rsp           VARCHAR2 (350);
      err_msg         VARCHAR2 (256);
      err_num         NUMBER;

      wnulls          EXCEPTION;
      er_iddupli      EXCEPTION;
      er_dates        EXCEPTION;
      es_histo        EXCEPTION;
      notfnd1         EXCEPTION;
      es_histo1       EXCEPTION;
      v_mitja         NUMBER;
      w_suptx100      NUMBER;
      v_num           NUMBER;
      v_num2           NUMBER;


      /*TAULES  PARLAMENTARI = PA. ; PERSONA PE.  ; DESPESA D. ; PARTIT PI. ; 
        PREPARAR EL CURSOR AMB LA SELECT     */
      CURSOR C_CNS_E IS
         SELECT   PA.IDPARLAMENTARI
                 ,PE.NOM
                 ,PI.NOMPARTIT
                 ,NVL (SUM (D.IMPORTTOTAL), 0) IMPORT_PARLAMENTARI
         FROM     parlamentari pa
                 ,DESPESA D
                 ,persona pe
                 ,PARTIT PI
         WHERE    PA.IDPARLAMENT = P_IDPARLAMENT
         AND      PA.IDPARLAMENTARI = D.IDPARLAMENTARID(+)
         AND      NVL (D.ACTIU, 'S') = 'S'
         AND      NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), P_ANY) = p_any
         AND      PE.IDPERSONA = PA.IDPERSONA
         AND      PA.IDPARTIT = PI.IDPARTIT
         GROUP BY IDPARLAMENTARI, NOM, NOMPARTIT
         HAVING   NVL (SUM (D.IMPORTTOTAL), 0) > v_mitja;
   BEGIN
      r_any := p_any;
      r_idparlament := p_idparlament;
      -- PREPARA LES DADES PER LA TAULA log
       p_nompro := 'Consulta_e';
      p_inlog :=
         r_any || ',    ' || p_idparlament ;
      p_outlog := 'OK';
      RSP := 'OK';
      
      --Controlar que els camps no siguin nuls
      IF (r_any IS NULL) OR (r_idparlament IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- validar les dades d'entrada ----
      IF r_any > 2000 AND r_any <= 2013 THEN
         NULL;
      ELSE
         RAISE er_dates;
      END IF;
    
      -- VALIDAR QUE EL PARLAMENT EXISTEIX I ESTA ACTIU
      w_actiu := PAQ_PARLAMENT.estatParlament (r_idparlament);

      IF w_actiu IS NULL THEN
         RAISE notfnd1;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo1;
      END IF;


      -- PRIMER CALCUL LA MITJANA DE DESPESA DELS PARLAMENTARIS DELS
      -- PARLAMENT A V_MITJA
      SELECT AVG (IMPORT_PARLAMENTARI)
      INTO   V_MITJA
      FROM   (SELECT   PA.IDPARLAMENTARI
                      ,NVL (SUM (D.IMPORTTOTAL), 0) IMPORT_PARLAMENTARI
              FROM     parlamentari pa, DESPESA D
              WHERE    PA.IDPARLAMENT = P_IDPARLAMENT
              AND      PA.IDPARLAMENTARI = D.IDPARLAMENTARID(+)
              AND      NVL (D.ACTIU, 'S') = 'S'
              AND      NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), P_ANY) = p_any
              GROUP BY IDPARLAMENTARI);
              



      -- ESCRIUT LA CAALERA DEL LLISTAT

      DBMS_OUTPUT.PUT_LINE (
         'CONSULTA E: Parlamentaris que superen la despesa mitja dels parlamentaris del al seu parlament '
         || P_IDPARLAMENT
         || '    L''ANY: '
         || P_ANY);
     DBMS_OUTPUT.PUT_LINE (
         'NOM PARLAMENTARI | PARTIT POLITIC | TOTAL DESPESES ANY | %PER DAMUNT MITJANA' 
        );
         DBMS_OUTPUT.PUT_LINE (
         '---------------------------------------------------------------------------- '
        );

              
      -- si la mitjana de despesa s 0 no fa el clcul. Si divideix per 0 dona error         
      IF V_MITJA > 0 THEN
         FOR V_CNS IN C_CNS_E LOOP
         -- CALCUL del  %   de despesa que el  parlamentari  ha superat
         --   respecta a la mitjana  de gasto del total de  parlamentaris ----
            w_suptx100 := (v_cns.import_PARLAMENTARI * 100) / V_MITJA;
         
            -- escriu la linia de detall
            V_NUM := V_CNS.IMPORT_PARLAMENTARI;
            v_num2:= w_suptx100;
            DBMS_OUTPUT.PUT_LINE (
               V_CNS.nom
            || ' | '
            || V_CNS.NOMpartit
            ||' | '
            || (trunc(v_num,2))
            ||' | '
            || (trunc(v_num2,2))
            || '%');
         END LOOP;
      ELSE
        DBMS_OUTPUT.PUT_LINE('La mitjana de despesa dels parlamentaris s 0');
      END IF;
      
      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: Consuta_E.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Consulta E erronia. El Parlament no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo1 THEN
         rsp := 'ERROR:  Consulta E  erronia. El Parlament  s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Consulta E. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
          p_nompro:= 'Consulta_e';
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: Consulta_E[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END CONSULTA_E;

   /* ********************************************************************************************************
   *  NOM: CONSULTA_F
   *  DESCRIPCIO:
   *        DONAT UN ANY,
   *               EL LLISTAT DE TOTS ELS PARLAMENTS AMB EL SEU ESTAT COMPTABLE, INCLOENT:
   *               NOM DEL PARLAMENT
   *               NOMBRE DE PARLAMENTARIS
   *               SUMA DE TOTES LES DESPESES PENDENTS DAPROVACI.
   *               SUMA DE TOTES LES DESPESES APROVADES.
   *               SUMA DE TOTES LES DESPESES PENDENTS DABONAMENT.
   *               SUMA DE TOTES LES DESPESES ABONADES.
   *       PROCESSAR SOLS LES DESPESES,  PARLAMENTS I PARLAMENTARIS ACTIUS (ACTIU='S')
   *  PARAMETERS:
   *  IN PUT -> P_ANY,
   *  OUTPUT -> RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***************************************************************************************************************/

   PROCEDURE CONSULTA_F (P_ANY IN VARCHAR2, RSP OUT VARCHAR2) IS
      r_any      VARCHAR2 (4);
      w_actiu    VARCHAR2 (1);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      p_rsp      VARCHAR2 (350);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
      V_NUM      NUMBER;

      wnulls     EXCEPTION;
      er_dates   EXCEPTION;



      /*TAULES  PARLAMENTARI = PA. ; PERSONA PE.  ; DESPESA D. ; PARTIT PI. ; 
        PREPARAR EL CURSOR AMB LA SELECT */

      CURSOR C_CNS_F IS
         SELECT PT.NOM
               ,PT.TOTALP
               ,NVL (
                   (SELECT SUM (D1.IMPORTTOTAL)
                    FROM   DESPESA D1
                    WHERE  D1.ACTIU = 'S'
                    AND    D1.IDPARLAMENTD = PT.IDPARLAMENT
                    AND    D1.IDESTAT = 1
                    AND    TO_CHAR (D1.DATADESPESA, 'YYYY') = p_any)
                  ,0)
                   AS PENDENTS_APROVACIO
               ,NVL (
                   (SELECT SUM (D2.IMPORTTOTAL)
                    FROM   DESPESA D2
                    WHERE  D2.ACTIU = 'S'
                    AND    D2.IDPARLAMENTD = PT.IDPARLAMENT
                    AND    D2.IDESTAT = 2
                    AND    TO_CHAR (D2.DATADESPESA, 'YYYY') = p_any)
                  ,0)
                   AS APROVADES
               ,NVL (
                   (SELECT SUM (D3.IMPORTTOTAL)
                    FROM   DESPESA D3
                    WHERE  D3.ACTIU = 'S'
                    AND    D3.IDPARLAMENTD = PT.IDPARLAMENT
                    AND    D3.IDESTAT = 3
                    AND    TO_CHAR (D3.DATADESPESA, 'YYYY') = p_any)
                  ,0)
                   AS PENDENTS_ABONAMENT
               ,NVL (
                   (SELECT SUM (D4.IMPORTTOTAL)
                    FROM   DESPESA D4
                    WHERE  D4.ACTIU = 'S'
                    AND    D4.IDPARLAMENTD = PT.IDPARLAMENT
                    AND    D4.IDESTAT = 4
                    AND    TO_CHAR (D4.DATADESPESA, 'YYYY') = p_any)
                  ,0)
                   AS ABONADES
         FROM   PARLAMENT PT
         WHERE  PT.ACTIU = 'S';
   BEGIN
      -- preparar les dades per LOG
      r_any := p_any;
      p_nompro := 'Consulta_F';
      p_inlog := r_any;

      p_outlog := 'OK';
      RSP := 'OK';

      --Controlar que els camps no siguin nuls
      IF (r_any IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- validar les dades d'entrada ----
      IF r_any > 2000 AND r_any <= 2013 THEN
         NULL;
      ELSE
           RAISE er_dates;
      END IF;


      -- ESCRIU LA CAPALERA DEL LLISTAT
      DBMS_OUTPUT.PUT_LINE (
         'CONSULTA F: Estat Comptable de tots el PARLAMENTS  L''ANY: '
         || P_ANY);
         
         DBMS_OUTPUT.PUT_LINE (
         'NOM PARLAMENT  | TOTAL PARLAMENTARIS | DESPESES: PENDENTS D''APROVACI | APROVADES | PENDENTS D''ABONAMENT | ABONADES'
         );
          DBMS_OUTPUT.PUT_LINE (
         '---------------------------------------------------------------------------------------------------------------------'
         );
      ---   executa el llistat de la select declarada  a la definici del cursor ---
      FOR V_CNS IN C_CNS_F LOOP
         DBMS_OUTPUT.PUT_LINE (
               V_CNS.NOM
            || ' | '
            || V_CNS.TOTALP
            || ' | '
            || V_CNS.PENDENTS_APROVACIO
            || ' | '
            || V_CNS.APROVADES
            || ' | '
            || V_CNS.PENDENTS_ABONAMENT
            || ' | '
            || V_CNS.ABONADES);
      END LOOP;
      
      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: Consuta_f.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Consulta F. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         p_nompro:= 'Consulta_f';
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: Consulta_F[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END CONSULTA_F;
END PAQ_CONSULTA;
/
