CREATE OR REPLACE PACKAGE                       PAQ_DESPESA as
/*   Procedement per donar d'alta una despesa */ 
PROCEDURE altaDespesa(
                          p_idcategoria       IN     INTEGER
                         ,p_datadespesa       IN     DATE
                         ,p_descripcio        IN     VARCHAR2
                         ,p_justificacio      IN     VARCHAR2
                         ,p_urlimatge         IN     VARCHAR2
                         ,p_dataestat         IN     DATE
                         ,p_idpagament        IN     VARCHAR2
                         ,p_importbrut        IN     NUMBER
                         ,p_tx100iva          IN     NUMBER
                         ,p_idproveidor       IN     INTEGER
                         ,p_idestat           IN     INTEGER
                         ,p_idparlamentd      IN     INTEGER
                         ,p_idparlamentarid   IN     INTEGER
                         ,p_idtipus           IN     INTEGER
                        ,RSP                    OUT VARCHAR2 );
 
  
   /****************************************************************************
     Funcio que retorna l'estat d'una Despesa 
     Si DESPESA.actiu ='S'
        podem realitzar operacions perqu la despesa esta activa
     Si DESPESA.actiu ='N'
        El Parlament s historic i no podem operar amb les seves dades.
    
  ****************************************************************************/
     
  
  FUNCTION estatDespesa (p_iddespesa IN INTEGER) RETURN VARCHAR2;
  
 /*  Procediment per donar de baixa una despesa.
 Canvia l'atribut actiu = 'N'. s una baixa lgica, no fsica */    

   
  PROCEDURE baixaDespesa (p_Iddespesa IN INTEGER , RSP  OUT VARCHAR2);
  
  
  /* Procediment per modificar les dades d'una despesa*/  
 
   PROCEDURE modificacioDespesa ( 
    p_iddespesa         IN  Integer
  , p_idcategoria       IN     Integer
  , p_datadespesa          IN  Date
  , p_descripcio          IN  VARCHAR2
  , p_justificacio        IN  VARCHAR2
  , p_urlimatge            IN  VARCHAR2
  , p_dataestat            IN  Date
  , p_idpagament          IN  VARCHAR2
  , p_importbrut          IN  number  
  , p_tx100iva            IN  number
  , p_idproveidor          IN  Integer
  , p_idestat             IN  Integer
  , p_idparlamentd      IN    Integer
  , p_idparlamentarid   IN    Integer
  , p_idtipus             IN  Integer
   , RSP                 OUT VARCHAR2); 
  

  
  
END PAQ_DESPESA;
/


CREATE OR REPLACE pACKAGE BODY                                                                                                                                                           PAQ_DESPESA as

/* **********************************************************************
   *  NOM:         ALTADESPESA
   *  DESCRIPCIO:  GESTIONA L'ALTA D'UNA DESPESA A LA TAULA DESPESA
   *               IDDESPESA ES AUTONUMERIC  I NO ES PASA PER PARAMETRES
   *               ACTIU S L'INDICADOR DEL ESTAT DEL REGISTRE A LA TAULA  I NO
   *               ES PASA PER PARAMETRES
   *               ACTIU SE CREA A 'S'
   *               ACTIU = 'S' EL REGISTRE S VIGENT. PODEM OERAR AMB EL REGISTRE
   *               ACTIU = 'N' EL REGISTRE S HISTRIC. NO PODEM OPERAR AMB EL REGISTRE
   *               CRIDA LA FUNCIO ALTALOG PER GRTAVAR EL RESULTAT DE L'EXECUCI.
   *               IMPORTTOTAL SE CALCULA SEGONS LES DADES DE IMPORTBRUT I TX100IVA
   *               SI LA DESPESA S D'UN PARLAMENTARI IDPARTID ES CREA AMB EL PARTIT AL QUE PERTANY
   *               EL PARLLAMENTARI EN EL MOMENT DE LA SEVA CREACI
   *  PARAMETERS:
   *
   *  IN PUT -> P_IDCATEGORIA , P_DATADESPESA , P_DESCRIPCIO , P_JUSTIFICACIO
   *           , P_URLIMATGE , P_DATAESTAT , P_IDPAGAMENT , P_IMPORTBRUT
   *           , P_TX100IVA , P_IDPROVEIDOR , P_IDESTAT , P_IDPARLAMENTD
   *           , P_IDPARLAMENTARID , P_IDTIPUS , P_IDPARTITD
   *
   *  OUTPUT -> RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***********************************************************************/
   PROCEDURE altaDespesa (p_idcategoria       IN     INTEGER
                         ,p_datadespesa       IN     DATE
                         ,p_descripcio        IN     VARCHAR2
                         ,p_justificacio      IN     VARCHAR2
                         ,p_urlimatge         IN     VARCHAR2
                         ,p_dataestat         IN     DATE
                         ,p_idpagament        IN     VARCHAR2
                         ,p_importbrut        IN     NUMBER
                         ,p_tx100iva          IN     NUMBER
                         ,p_idproveidor       IN     INTEGER
                         ,p_idestat           IN     INTEGER
                         ,p_idparlamentd      IN     INTEGER
                         ,p_idparlamentarid   IN     INTEGER
                         ,p_idtipus           IN     INTEGER
                         ,RSP                    OUT VARCHAR2)     IS
                         
      r_iddespesa         INTEGER;
      r_idcategoria       INTEGER;
      r_datadespesa       DATE;
      r_descripcio        DESPESA.DESCRIPCIO%TYPE;-- VARCHAR2 (60);
      r_justificacio      DESPESA.JUSTIFICACIO%TYPE;-- VARCHAR2 (60);
      r_urlimatge         DESPESA.URLIMATGE%TYPE; --VARCHAR2 (50);
      r_dataestat         DATE;
      r_idpagament        DESPESA.IDPAGAMENT%TYPE;--VARCHAR2 (25);
      r_importbrut        NUMBER;
      r_tx100iva          NUMBER;
      r_importtotal       NUMBER;
      r_idproveidor       INTEGER;
      r_idestat           INTEGER;
      r_idparlamentd      INTEGER;
      r_idparlamentarid   INTEGER;
      r_idpartitd         INTEGER;
      r_idtipus           INTEGER;
      r_idpartit          INTEGER;
      r_actiu             VARCHAR2 (1);


      w_actiu             VARCHAR2 (1);
      p_nompro            VARCHAR2 (50);
      p_inlog             VARCHAR2 (500);
      p_outlog            VARCHAR2 (500);
      p_rsp               VARCHAR2 (350);
      err_msg             VARCHAR2 (256);
      err_num             NUMBER;
      w_data              DATE;
      w_modul             VARCHAR2 (2);


      wnulls              EXCEPTION;
      er_iddupli          EXCEPTION;
      er_dates            EXCEPTION;
      er_dates1           EXCEPTION;
      er_dates2           EXCEPTION;
      er_estad            EXCEPTION;
      es_histo1           EXCEPTION;
      es_histo2           EXCEPTION;
      notfnd1             EXCEPTION;
      notfnd2             EXCEPTION;
      notfnd3             EXCEPTION;
      notfnd4             EXCEPTION;
      notfnd5             EXCEPTION;
      notfnd6             EXCEPTION;
   BEGIN
      p_nompro := 'alta despesa';
      r_idcategoria := p_idcategoria;
      r_datadespesa := p_datadespesa;
      r_descripcio := SUBSTR (p_descripcio, 1, 60);
      r_justificacio := SUBSTR (p_justificacio, 1, 60);
      r_urlimatge := SUBSTR (p_urlimatge, 1, 50);
      r_dataestat := p_dataestat;
      r_idpagament := SUBSTR (p_idpagament, 1, 50);
      r_importbrut := p_importbrut;
      r_tx100iva := p_tx100iva;
      r_importtotal := 0;
      r_idproveidor := p_idproveidor;
      r_idestat := p_idestat;
      r_idparlamentd := p_idparlamentd;
      r_idparlamentarid := p_idparlamentarid;
      r_idtipus := p_idtipus;
      r_actiu := 'S';
-- GUARDAR LES DADES PER ACTUALITZAR LA TAULA LOG
      
      p_inlog :=
            r_iddespesa
         || ',    '
         || p_idcategoria
         || ',    '
         || p_datadespesa
         || ',    '
         || p_descripcio
         || ',    '
         || p_justificacio
         || ',    '
         || p_urlimatge
         || ',    '
         || p_dataestat
         || ',    '
         || p_idpagament
         || ',    '
         || p_importbrut
         || ',    '
         || p_tx100iva
         || ',    '
         || r_importtotal
         || ',    '
         || p_idproveidor
         || ',    '
         || p_idestat
         || ',    '
         || p_idparlamentd
         || ',    '
         || p_idparlamentarid
         || ',    '
         || p_idtipus
         || ',    '
         || r_idpartitd
         || ',    '
         || r_actiu;
      p_outlog := 'OK';
      RSP := 'OK';

      --Controlar que els camps no siguin nuls
      IF (r_idcategoria IS NULL)
         OR (r_datadespesa IS NULL)
         OR (r_descripcio IS NULL)
         OR (r_justificacio IS NULL)
         OR (r_urlimatge IS NULL)
         OR (r_dataestat IS NULL)
         OR (r_importbrut IS NULL)
         OR (r_tx100iva IS NULL)
         OR (r_idproveidor IS NULL)
         OR (r_idparlamentd IS NULL)
         OR (r_idestat IS NULL)
         OR (r_idtipus IS NULL) THEN
         RAISE wnulls;
      END IF;
      

      -- validar que les dades de categoria siguin correctes
      w_actiu := PAQ_UTI.hiHaCategoria (p_idcategoria);

      IF w_actiu = 'S' THEN null; else
         RAISE notfnd1;
      END IF;

      w_actiu := 'N';

      -- si categoria = 0 ha de tenir idparlament valid
      -- si categoria = 1 ha de tenir idparlamentari valid.
      IF ( (r_idcategoria = 0
            AND r_idparlamentd IS NOT NULL
            AND r_idparlamentarid IS NULL)
          OR (r_idcategoria = 1
              AND r_idparlamentd IS not NULL
              AND r_idparlamentarid IS NOT NULL)) THEN
         w_actiu := 'S';
      END IF;

      IF w_actiu = 'N' THEN
         RAISE er_iddupli;
      END IF;
    -- validar que el codi de  proveidor s correcte
      w_actiu := PAQ_UTI.hiHaProveidor (r_idproveidor);

      IF w_actiu = 'S' THEN null; else
         RAISE notfnd2;
      END IF;
    -- valia que el codi d'estat de la despesa s correcte
      w_actiu := PAQ_UTI.hiHaEstatDespesa (r_idestat);

       IF w_actiu = 'S' THEN null; else
         RAISE notfnd3;
      END IF;
  -- valida si el codi de parlament s correcte
       w_actiu := PAQ_PARLAMENT.estatParlament (r_idparlamentd);
        IF w_actiu = 'N' THEN
            RAISE es_histo1;
         END IF;


          IF w_actiu = 'S' then null; else
            RAISE notfnd4;
         END IF;

        
      IF r_idcategoria = 1       THEN
       -- si la categori s 1 valida el codi de parlamentari
         w_actiu := PAQ_PARLAMENTARI.estatParlamentari (r_idparlamentarid);

        IF w_actiu = 'N' THEN
            RAISE es_histo2;
         END IF;
         
          IF w_actiu = 'S' then null; else
            RAISE notfnd5;
         END IF;

      END IF;
    -- valida el codi del tipus de despesa
      w_actiu := PAQ_UTI.hiHaTipusDespesa (p_idtipus);

      IF w_actiu = 'S' then null; else
         RAISE notfnd6;
      END IF;
   -- valida que la data de despessa sigui anterior a la data del dia.
      IF (TRUNC (r_datadespesa) > TRUNC (SYSDATE)) THEN
         RAISE er_dates;
      END IF;

      -- si categoria = 0 es despesa de parlament ha de tenir codi banc
      -- si categoria = 1 es despesa de parlamentari  ha de tenir codi visa i idpartit

      IF r_idcategoria = 0 THEN
         SELECT idbanc, datacreacio
         INTO   r_idpagament, w_data
         FROM   PARLAMENT
         WHERE  (IDPARLAMENT = r_idparlamentd);
-- validar que la data de despesa no es inferior a la creacio del Parlament
         IF TRUNC (r_datadespesa) < TRUNC (w_data) THEN
            RAISE er_dates1;
         END IF;
      ELSE
      
         
         SELECT idvisa, idpartit, dataalta
         INTO   r_idpagament, r_idpartitd, w_data
         FROM   PARLAMENTARI
         WHERE  (IDPARLAMENTARI = r_idparlamentarid);
       
-- valida que la data despesa es superior a la data alta parlamentari
 
         IF TRUNC (r_datadespesa) < TRUNC (w_data) THEN
            RAISE er_dates2;
         END IF;
      END IF;
   
      
      
  -- calcula l'import total de la despesa
      r_importtotal := r_importbrut + ((r_importbrut * r_tx100iva) / 100);

      -- obtenir valor autonumeric per alta de despesa
      SELECT S_DESPESA.NEXTVAL INTO r_idDESPESA FROM DUAL;

      INSERT INTO DESPESA (iddespesa
                          ,idcategoria
                          ,datadespesa
                          ,descripcio
                          ,justificacio
                          ,urlimatge
                          ,dataestat
                          ,idpagament
                          ,importbrut
                          ,tx100iva
                          ,importtotal
                          ,idproveidor
                          ,idestat
                          ,idparlamentd
                          ,idparlamentarid
                          ,idtipus
                          ,idpartitd
                          ,actiu)
      VALUES      (r_iddespesa
                  ,r_idcategoria
                  ,r_datadespesa
                  ,r_descripcio
                  ,r_justificacio
                  ,r_urlimatge
                  ,r_dataestat
                  ,r_idpagament
                  ,r_importbrut
                  ,r_tx100iva
                  ,r_importtotal
                  ,r_idproveidor
                  ,r_idestat
                  ,r_idparlamentd
                  ,r_idparlamentarid
                  ,r_idtipus
                  ,r_idpartitd
                  ,r_actiu);

      -- ACTUALITZAR MODUL ESTADISTIC E1
      PAQ_EST.modul_E1 (RSP);
      w_modul := 'E1';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E2
      PAQ_EST.modul_E2 (r_idparlamentd, RSP);
      w_modul := 'E2';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E3
      PAQ_EST.modul_E3 (RSP);
      w_modul := 'E3';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E4
      PAQ_EST.modul_E4 (RSP);
      w_modul := 'E4';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E5
      PAQ_EST.modul_E5 (RSP);
      w_modul := 'E5';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E6
      PAQ_EST.modul_E6 (RSP);
      w_modul := 'E6';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E7
      PAQ_EST.modul_E7 (RSP);
      w_modul := 'E7';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E8
      PAQ_EST.modul_E8 (RSP);
      w_modul := 'E8';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      --------------------------------------------
      /* si totes les execucions sn correctes confirmem */
      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- alta Despesa  realitzada --');
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_iddupli THEN
         rsp :=
            'ERROR: Alta erronia. Clau Despesa no es correpon amb la categoria';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Alta erronia. La Categoria no existeix existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd2 THEN
         rsp := 'ERROR: Alta erronia. El Proveidor  no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd3 THEN
         rsp := 'ERROR: Alta erronia. Estat de la despesa no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd4 THEN
         rsp := 'ERROR: Alta erronia. El Parlament  no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
       
      WHEN es_histo1 THEN
         rsp := 'ERROR: Alta erronia. El Parlament s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd5 THEN
         rsp := 'ERROR: Alta erronia. El Parlamentari  no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo2 THEN
         rsp := 'ERROR: Alta erronia. El Parlamentari s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd6 THEN
         rsp := 'ERROR: Alta erronia. El Tipus de Despesa  no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Alta erronia. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         WHEN er_dates1 THEN
         rsp := 'ERROR: Alta erronia. El camps amb dates sn  erronis amb dades parlament';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         WHEN er_dates2 THEN
         rsp := 'ERROR: Alta erronia. El camps amb dates sn  erronis amb dades parlamentari';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_estad THEN
         rsp := 'No actualitza: Modul ' || w_modul || ' Alta despesa';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         rsp := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END altaDespesa;

   /***********************************************************************
   *  NOM:         ESTATDESPESA
   *  DESCRIPCIO:  SERVEIX PER SABER SI UNA DESPESA  EXISTEIX I SI ESTA ACTIUVA O  NO
   *                      CRIDA LA FUNCIO ALTALOG SI L'EXECUCI S ERRONIA
   *  PARAMETERS:
   *  INPUT ->     P_IDDESPESA
   *  OUTPUT ->    P_ ACTIU
   *                   RETORNA:
   *                   SI  EL PARLAMENT NO EXISTEIX RETORNA NULLS A P_ACTIU
   *                   SI EL PARLAMENT EXISTEIX RETORNA  EL VALOR DEL CAMP ACTIU
   ***********************************************************************/
   FUNCTION estatdespesa (p_iddespesa IN INTEGER)
      RETURN VARCHAR2 IS
      RSP        VARCHAR2 (50);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (100);
      p_outlog   VARCHAR2 (100);
      p_rsp      VARCHAR2 (50);
      err_msg    VARCHAR2 (50);
      err_num    NUMBER;
      w_actiu    VARCHAR2 (1);
   BEGIN
      p_nompro := 'estatDespesa';
      p_inlog := p_iddespesa;

      SELECT ACTIU
      INTO   w_actiu
      FROM   DESPESA
      WHERE  (IDDESPESA = p_iddespesa);

      p_outlog := w_actiu;
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      RETURN w_actiu;                      
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: La despesa  no existeix';
         w_actiu := NULL;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         RETURN null;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         RETURN null;
   END estatDespesa;


   /* **********************************************************************
   *  NOM:        BAIXADESPESA
   *  DESCRIPCIO:  GESTIONA LA BAIXA D'UNA DESPESA A LA TAULA DESPESA .
   *                LA BAIXA S LGICA. PER PODER  MANTENIR TOTA LA INFORMACI DE  L'HISTORIC DE DESPESA
   *               CRIDA  A LA FUNCIO ESTATDESPESA.
   *              SI EL PARLAMENT EXISTEIX I ACTIU = 'S'
   *                 MODIFIQUEM PARLAMENT AMB ACTIU = 'N'
   *                MODIFIQUEM TOTS LES ESTADISTIQUES
   *               CRIDA LA FUNCIO ALTALOG PER GRTAVAR EL RESULTAT DE L'EXECUCI.
   *  PARAMETERS:
   *  INPUT ->   P_IDDESPESA
   *  OUTPUT ->
   *             RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ********************************************************************** */


   PROCEDURE baixaDespesa (p_Iddespesa IN INTEGER, rsp OUT VARCHAR2) IS
      r_idparlamentd   INTEGER;
      r_importtotal    INTEGER;
      r_datadespesa    DATE;
      w_actiu          VARCHAR2 (1);
      p_nompro         VARCHAR2 (50);
      p_inlog          VARCHAR2 (100);
      p_outlog         VARCHAR2 (100);
      p_rsp            VARCHAR2 (50);
      err_msg          VARCHAR2 (50);
      err_num          NUMBER;
      w_modul          VARCHAR2 (2);


      wnulls           EXCEPTION;
      es_histo         EXCEPTION;
      notfnd           EXCEPTION;
      er_estad         EXCEPTION;
   BEGIN
      --S'emmagatzema els parmetres d'entrada per actualitzar el LOG
      p_nompro := 'Baixa despesa';
      p_inlog := p_iddespesa || ', ' || w_actiu;
      p_outlog := ' ';


      IF p_iddespesa IS NULL THEN
         RAISE wnulls;
      END IF;
    -- valida que la despesa existeix i que actiu = 'S'
      w_actiu := estatdespesa (p_iddespesa);

      IF w_actiu IS NULL THEN
         RAISE notfnd;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo;
      END IF;

-- la baixa logica modifica el camp actiu = 'N'
      UPDATE despesa
      SET    ACTIU = 'N'
      WHERE  IDdespesa = p_iddespesa;

      -- ACTUALITZAR MODUL ESTADISTIC E1
      PAQ_EST.modul_E1 (RSP);
      w_modul := 'E1';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E2
      PAQ_EST.modul_E2 (r_idparlamentd, RSP);
      w_modul := 'E2';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E2
      PAQ_EST.modul_E3 (RSP);
      w_modul := 'E3';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E4
      PAQ_EST.modul_E4 (RSP);
      w_modul := 'E4';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E5
      PAQ_EST.modul_E5 (RSP);
      w_modul := 'E5';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E6
      PAQ_EST.modul_E6 (RSP);
      w_modul := 'E6';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E7
      PAQ_EST.modul_E7 (RSP);
      w_modul := 'E7';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      -- ACTUALITZAR MODUL ESTADISTIC E8
      PAQ_EST.modul_E8 (RSP);
      w_modul := 'E8';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      --------------------------------------------
      /* si totes les execucions sn correctes confirmem */
      ------------------------------------------------------------------
      rsp := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- baixa despesa realitzada --');
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: baixa.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd THEN
         rsp := 'ERROR: Baixa erronia. La despesa no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo THEN
         rsp := 'ERROR: Baixa erronia. La despesa s Histrica';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_estad THEN
         rsp := 'Error Modul ' || w_modul || ' Baixa despesa';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR:baixa.[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END baixaDespesa;

   /* **********************************************************************
   *  NOM:         MODIFICACIODESPESA
   *  DESCRIPCIO:  GESTIONA LA MODIFICACIO  D'UNA DESPESA A LA TAULA DESPESA
   *               CRIDA  A LA FUNCIO ESTATDESPESA.
   *               SI  LA DESPESA  EXISTEIX I ACTIU = 'S'
   *                   VALIDAR LES DADES I SI SN CORRECTES MODIFICAR LA DESPESA
   *               IMPORTTOTAL SE CALCULA SEGONS LES DADES DE IMPORTBRUT I TX100IVA
   *  PARAMETERS:
   *  INPUT ->  P_IDDESPESA
   *              P_IDCATEGORIA , P_DATADESPESA , P_DESCRIPCIO , P_JUSTIFICACIO
   *           , P_URLIMATGE , P_DATAESTAT , P_IDPAGAMENT , P_IMPORTBRUT
   *           , P_TX100IVA , P_IDPROVEIDOR , P_IDESTAT , P_IDPARLAMENTD
   *           , P_IDPARLAMENTARID , P_IDTIPUS ,
   *   OUTPUT ->
   *             RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ********************************************************************** */

   PROCEDURE modificacioDespesa ( 
   p_iddespesa         IN  Integer
  , p_idcategoria       IN     Integer
  , p_datadespesa          IN  Date
  , p_descripcio          IN  VARCHAR2
  , p_justificacio        IN  VARCHAR2
  , p_urlimatge            IN  VARCHAR2
  , p_dataestat            IN  Date
  , p_idpagament          IN  VARCHAR2
  , p_importbrut          IN  number  
  , p_tx100iva            IN  number
  , p_idproveidor          IN  Integer
  , p_idestat             IN  Integer
  , p_idparlamentd      IN    Integer
  , p_idparlamentarid   IN    Integer
  , p_idtipus             IN  Integer
    , RSP                 OUT VARCHAR2) IS
      r_iddespesa         INTEGER;
      r_idcategoria       INTEGER;
      r_datadespesa       DATE;
      r_descripcio        DESPESA.DESCRIPCIO%TYPE;
      r_justificacio      DESPESA.JUSTIFICACIO%TYPE;
      r_urlimatge         DESPESA.URLIMATGE%TYPE;
      r_dataestat         DATE;
      r_idpagament        DESPESA.IDPAGAMENT%TYPE;
      r_importbrut        NUMBER;
      r_tx100iva          NUMBER;
      r_importtotal       NUMBER;
      r_idproveidor       INTEGER;
      r_idestat           INTEGER;
      r_idparlamentd      INTEGER;
      r_idparlamentarid   INTEGER;
      r_idpartitd         INTEGER;
      r_idtipus           INTEGER;
      r_actiu             VARCHAR2 (1);
      w_idparlamentd      INTEGER;
      w_datadespesa       DATE;
      w_importtotal       INTEGER;

      w_actiu             VARCHAR2 (1);
      p_nompro            VARCHAR2 (50);
      p_inlog             VARCHAR2 (500);
      p_outlog            VARCHAR2 (500);
      p_rsp               VARCHAR2 (350);
      err_msg             VARCHAR2 (256);
      err_num             NUMBER;
      w_data              DATE;
      w_modul             VARCHAR2 (2);


      wnulls              EXCEPTION;
      er_iddupli          EXCEPTION;
      er_dates            EXCEPTION;
      er_dates1           EXCEPTION;
      er_dates2           EXCEPTION;
      es_histo            EXCEPTION;
      es_histo1           EXCEPTION;
      es_histo2           EXCEPTION;
      notfnd              EXCEPTION;
      notfnd1             EXCEPTION;
      notfnd2             EXCEPTION;
      notfnd3             EXCEPTION;
      notfnd4             EXCEPTION;
      notfnd5             EXCEPTION;
      notfnd6             EXCEPTION;
      er_estad            EXCEPTION;

      v_modest            VARCHAR2 (1);
   BEGIN
      p_nompro := 'Modificaci despesa';
      r_iddespesa := p_iddespesa;
      r_idcategoria := p_idcategoria;
      r_datadespesa := p_datadespesa;
      r_descripcio := SUBSTR (p_descripcio, 1, 60);
      r_justificacio := SUBSTR (p_justificacio, 1, 60);
      r_urlimatge := SUBSTR (p_urlimatge, 1, 50);
      r_dataestat := p_dataestat;
      r_idpagament := SUBSTR (p_idpagament, 1, 50);
      r_importbrut := p_importbrut;
      r_tx100iva := p_tx100iva;
      r_importtotal := 0;
      r_idproveidor := p_idproveidor;
      r_idestat := p_idestat;
      r_idparlamentd := p_idparlamentd;
      r_idparlamentarid := p_idparlamentarid;
      r_idtipus := p_idtipus;
     
      -- preparar les dades per actualitzar taula LOG

      
      p_inlog := 
            p_iddespesa
         || ',    '
         || p_idcategoria
         || ',    '
         || p_datadespesa
         || ',    '
         || p_descripcio
         || ',    '
         || p_justificacio
         || ',    '
         || p_urlimatge
         || ',    '
         || p_dataestat
         || ',    '
         || p_idpagament
         || ',    '
         || p_importbrut
         || ',    '
         || p_tx100iva
         || ',    '
         || r_importtotal
         || ',    '
         || p_idproveidor
         || ',    '
         || p_idestat
         || ',    '
         || p_idparlamentd
         || ',    '
         || p_idparlamentarid
         || ',    '
         || p_idtipus;

      p_outlog := 'OK';
      RSP := 'OK';

      --Controlar que els camps no siguin nuls
      IF (r_idcategoria IS NULL)
         OR (r_datadespesa IS NULL)
         OR (r_descripcio IS NULL)
         OR (r_justificacio IS NULL)
         OR (r_urlimatge IS NULL)
         OR (r_dataestat IS NULL)
         OR (r_importbrut IS NULL)
         OR (r_tx100iva IS NULL)
         OR (r_idproveidor IS NULL)
         OR (r_idestat IS NULL)
         OR (r_idtipus IS NULL) THEN
         RAISE wnulls;
      END IF;
    -- validar que el codi  del estat de la despesa es correcte
      -- valida que la despesa existeix i que actiu = 'S'
      w_actiu := estatdespesa (p_iddespesa);

      IF w_actiu IS NULL THEN
         RAISE notfnd;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo;
      END IF;

      -- validar que el codi de la categoria de la despesa es correcte
      w_actiu := PAQ_UTI.hiHaCategoria (p_idcategoria);

      IF w_actiu = 'S' THEN null; else
         RAISE notfnd1;
      END IF;

    
        
    -- validar que el codi de proveidor es correcte
      w_actiu := PAQ_UTI.hiHaProveidor (r_idproveidor);

     IF w_actiu = 'S' THEN null; else
         RAISE notfnd2;
      END IF;
    -- validar que el codi del estat de la despesa es correcte
      w_actiu := PAQ_UTI.hiHaEstatDespesa (r_idestat);

      IF w_actiu = 'S' THEN null; else
         RAISE notfnd3;
      END IF;
    
    -- validar que el codi de tipus de despesa s correcte
      w_actiu := PAQ_UTI.hiHaTipusDespesa (p_idtipus);

      IF w_actiu = 'S' THEN null; else
         RAISE notfnd6;
      END IF;
    -- validar que les dates de despesa i darrer estat no son superiors a la del dia
      IF (TRUNC (r_datadespesa) > TRUNC (SYSDATE)) THEN
         RAISE er_dates;
      END IF;

      IF (TRUNC (r_dataestat) > TRUNC (SYSDATE)) THEN
         RAISE er_dates;
      END IF;

      -- si categoria = 0 es despesa de parlament ha de tenir codi banc
      -- si categoria = 1 es despesa de parlamentari  ha de tenir codi visa i idpartit

      IF r_idcategoria = 0 THEN
         SELECT idbanc, datacreacio
         INTO   r_idpagament, w_data
         FROM   PARLAMENT
         WHERE  (IDPARLAMENT = r_idparlamentd);
         -- validar que la data de despesa   no es inferior a la data de creacio del parlament
         IF TRUNC (r_datadespesa) < TRUNC (w_data) THEN
            RAISE er_dates1;
         END IF;
      ELSE
         SELECT idvisa,  dataalta
         INTO   r_idpagament,  w_data
         FROM   PARLAMENTARI
         WHERE  (IDPARLAMENTARI = r_idparlamentarid);
-- validar que la data de despesa   no es inferior a la data d'alta del parlamentari

         IF TRUNC (r_datadespesa) < TRUNC (w_data) THEN
            RAISE er_dates2;
         END IF;
      END IF;
      
      
    -- calcular el import total de la despesa
      r_importtotal := r_importbrut + ((r_importbrut * r_tx100iva) / 100);

      
    -- comprovar si hi ha que modificar el mduls estadistics. 
      v_modest :=
         paq_uti.calcularEstadistica (r_datadespesa
                                     ,r_importbrut
                                     ,r_tx100iva
                                     ,p_iddespesa);
    -- modifiquem el registre segons els parametres passats
      UPDATE DESPESA
      SET    idcategoria = r_idcategoria
            ,datadespesa = r_datadespesa
            ,descripcio = r_descripcio
            ,justificacio = r_justificacio
            ,urlimatge = r_urlimatge
            ,dataestat = r_dataestat
            ,idpagament = r_idpagament
            ,importbrut = r_importbrut
            ,tx100iva = r_tx100iva
            ,importtotal = r_importtotal
            ,idproveidor = r_idproveidor
            ,idestat = r_idestat
            ,idtipus = r_idtipus
            WHERE  iddespesa = p_iddespesa;
-- si s'han modificat les dades importbrut, tx100iva o datadespesa modifiquem els mduls estadistics.
      IF v_modest = 'S' THEN
         -- ACTUALITZAR MODUL ESTADISTIC E1
         PAQ_EST.modul_E1 (RSP);
         w_modul := 'E1';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;

         -- ACTUALITZAR MODUL ESTADISTIC E2
         PAQ_EST.modul_E2 (r_idparlamentd, RSP);
         w_modul := 'E2';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;

         -- ACTUALITZAR MODUL ESTADISTIC E3
         PAQ_EST.modul_E3 (RSP);
         w_modul := 'E3';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;

         -- ACTUALITZAR MODUL ESTADISTIC E4
         PAQ_EST.modul_E4 (RSP);
         w_modul := 'E4';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;

         -- ACTUALITZAR MODUL ESTADISTIC E5
         PAQ_EST.modul_E5 (RSP);
         w_modul := 'E5';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;

         -- ACTUALITZAR MODUL ESTADISTIC E6
         PAQ_EST.modul_E6 (RSP);
         w_modul := 'E6';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;

         -- ACTUALITZAR MODUL ESTADISTIC E7
         PAQ_EST.modul_E7 (RSP);
         w_modul := 'E7';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;

         -- ACTUALITZAR MODUL ESTADISTIC E8
         PAQ_EST.modul_E8 (RSP);
         w_modul := 'E8';

         IF RSP <> 'OK' THEN
            RAISE er_estad;
         END IF;
      END IF;

      --------------------------------------------
      /* si totes les execucions sn correctes confirmem */
      ------------------------------------------------------------------

      p_outlog := 'OK';
      RSP := 'OK'; 
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- Modificacio Despesa realitzada --');
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: modif.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         
         WHEN notfnd THEN
         rsp := 'ERROR: Modificacio erronia. La despesa no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo THEN
         rsp := 'ERROR: Modificacio  erronia. La despesa s Histrica';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
           
      WHEN notfnd1 THEN
         rsp := 'ERROR: Modificacio erronia. La  Despesa no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo1 THEN
         rsp := 'ERROR: Modificacio erronia. La Despesa s Histrica';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
                
      WHEN notfnd2 THEN
         rsp := 'ERROR: Modificacio erronia. El proveidor no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd3 THEN
         rsp := 'ERROR: Modificacio erronia. Estat despesa  no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd6 THEN
         rsp := 'ERROR: Modificacio erronia. El Tipud de despesa no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp :=
            'ERROR: Modificaci erronia. Els camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         WHEN er_dates1 THEN
         rsp :=
            'ERROR: Modificaci erronia. data de despesa < data creacio Parlament';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         WHEN er_dates2 THEN
         rsp :=
            'ERROR: Modificaci erronia. data de despesa < data alta Parlamentari';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_estad THEN
         rsp := 'No actualitza: Modul ' || w_modul || ' Modificaci despesa';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modif.[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modificacioDespesa;
END PAQ_DESPESA;
/
