CREATE OR REPLACE PACKAGE 
                                    PAQ_EST IS
/* modul_E1:
Calcula per cada parlament la suma de totes les despeses dels darrers 4 anys 
(incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
*/                              
PROCEDURE modul_E1 (RSP OUT VARCHAR2);
/* modul_E2:
Per cada  any amb despeses asociades  calcula per cada parlament la diferencia en euros entre el parlamentari
que ms despeses associades t i el que menys.
*/   
PROCEDURE modul_E2 (p_parlament  Integer, RSP  OUT VARCHAR2); 
/* modul_E3:
Per cada any amb despeses asociades  alcula el parlament que ha tingut un total de despeses
ms alt (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
*/   
PROCEDURE modul_E3 (RSP OUT VARCHAR2);
/* modul_E4:
Calcula per cada  paralmentari la despesa total que ha tingu els darrers 4 anys */   
PROCEDURE modul_E4 (RSP OUT VARCHAR2);
/* modul_E5:
Per cada any amb despeses asociades calcula la mitjana de despesa dun parlamentari durant aquell any 
(tenint en compte tots els parlamentaris de tots els parlaments).*/   
PROCEDURE modul_E5 (RSP OUT VARCHAR2);
/* modul_E6:
Per cada any amb despeses asociades de parlamentaris calcula el parlamentari que ha tingut ms despesa associada
i el que menys (tenint en compte tots els parlaments).*/   
PROCEDURE modul_E6 (RSP OUT VARCHAR2);
/* modul_E7:
Per cada any amb despeses asociades  calcula  el nom del parlament amb 
una mitjana de despesa associada als seus parlamentaris ms alta, 
i el valor daquesta mitjana de despesa.*/   
PROCEDURE modul_E7 (RSP OUT VARCHAR2);
/* modul_E8:
Per cada any amb despeses asociades calcula la suma de totes les despeses de tots els parlaments
(incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).*/   
PROCEDURE modul_E8 (RSP OUT VARCHAR2);
end paq_est;
/


CREATE OR REPLACE PACKAGE BODY                                                                                      PAQ_EST IS
  /* **********************************************************************************************************
  *  NOM:         MODUL_E1
  *  DESCRIPCIO: ACTUALITZAR LA TAULA E1 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *             	Calcula per cada parlament la suma de totes les despeses dels darrers 4 anys 
  *               (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
  *  PARAMETERS:
  *  IN PUT ->  NO HI PARAMETRES D'ENTRADA PERQUE S RECALCULA TOTA LA TAULA.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/
   PROCEDURE modul_E1 (RSP OUT VARCHAR2) IS
      v_any      VARCHAR2 (4);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
      v_import   NUMBER;
      v_num       NUMBER;

      CURSOR c_pt IS
         SELECT IDPARLAMENT FROM PARLAMENT;
   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E1';
      p_inlog := '';
      v_any := TO_CHAR (SYSDATE, 'YYYY');

      DELETE FROM E1;

      FOR V_PT IN C_PT LOOP
         SELECT NVL (SUM (IMPORTTOTAL), 0)
         INTO   v_import
         FROM   DESPESA
         WHERE  ACTIU = 'S'
         AND    IDPARLAMENTD = v_pt.IDPARLAMENT
         AND    TO_CHAR (DATADESPESA, 'YYYY') BETWEEN v_any - 4 AND v_any;
         
         INSERT INTO E1 (IDPARLAMENT, RESULTAT)
         VALUES      (v_pt.IDPARLAMENT, (trunc(v_import,2)));
      END LOOP;

      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E1[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modul_E1;
   
  /* **********************************************************************************************************
  *  NOM:         MODUL_E2
  *  DESCRIPCIO: ACTUALITZAR TAULA E2 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *              Per cada  any amb despeses asociades  calcula per cada parlament la diferencia en euros entre el parlamentari
  *              que ms despeses associades t i el que menys.
  *  PARAMETERS:
  *  IN PUT ->  P_PARLAMENT.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/
   PROCEDURE modul_E2 (p_parlament INTEGER, RSP OUT VARCHAR2) IS
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
      v_num       NUMBER;


      -- modificarem el registre que es corresponen al parlament del
      --parlamentari de la despesa i a l'anys de la data de la despesa

      -- cridam des de ABM despeses de parlamentaris
      -- Alta de  parlamentaris per tenir en compte la despesa 0

      CURSOR c_any IS
         SELECT   TO_CHAR (D.DATADESPESA, 'YYYY') AS ANYD
         FROM     despesa d
         WHERE    d.actiu = 'S' AND D.IDPARLAMENTD = p_parlament
         GROUP BY TO_CHAR (D.DATADESPESA, 'YYYY');

      CURSOR C_E2 (
         p_anydespesa VARCHAR2) IS
         SELECT   PA.IDPARLAMENTARI
                 ,NVL (SUM (D.IMPORTTOTAL), 0) AS IMPORTTOTAL
         FROM     despesa d, PARLAMENTARI PA
         WHERE    NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), p_anydespesa) =
                     p_anydespesa
         AND      NVL (D.ACTIU, 'S') = 'S'
         AND      D.IDPARLAMENTARID(+) = PA.IDPARLAMENTARI
         AND      PA.IDPARLAMENT = p_parlament
         GROUP BY IDPARLAMENTARI;

      v_impmax   NUMBER := 0;
      v_impmin   NUMBER := 0;
      v_primer   NUMBER := 1;        -- insica si es el primer registre trobat
   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E2';
      p_inlog := p_parlament;

      DELETE FROM e2
      WHERE       idparlament = p_parlament;

      FOR v_any IN c_any LOOP
         FOR V_E2 IN C_E2 (V_ANY.ANYD) LOOP
            IF v_primer = 1 THEN
               v_impmin := v_e2.IMPORTTOTAL;
               v_primer := 0;
            END IF;

            IF v_e2.IMPORTTOTAL > v_impmax THEN
               v_impmax := v_e2.IMPORTTOTAL;
            END IF;

            IF v_e2.IMPORTTOTAL < v_impmin THEN
               v_impmin := v_e2.IMPORTTOTAL;
            END IF;
         END LOOP;

         INSERT INTO e2 (IDANY, IDPARLAMENT, RESULTAT)
         VALUES      (
                        TO_NUMBER (V_ANY.ANYD)
                       ,p_parlament
                       ,v_impmax - v_impmin);
      END LOOP;


      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E2[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modul_E2;
   
 /* **********************************************************************************************************
  *  NOM:         MODUL_E3
  *  DESCRIPCIO: ACTUALITZAR TAULA E3 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *              Per cada any amb despeses asociades  alcula el parlament que ha tingut un total de despeses
  *              ms alt (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
  *  PARAMETERS:
  *  IN PUT ->  NO HI PARAMETRES D'ENTRADA PERQUE S RECALCULA TOTA LA TAULA.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/


   PROCEDURE modul_E3 (RSP OUT VARCHAR2) IS
      p_nompro        VARCHAR2 (50);
      p_inlog         VARCHAR2 (500);
      p_outlog        VARCHAR2 (500);
      err_msg         VARCHAR2 (256);
      err_num         NUMBER;
       v_num       NUMBER;

      CURSOR c_any IS
         SELECT   TO_CHAR (D.DATADESPESA, 'YYYY') AS ANYD
         FROM     despesa d
         WHERE    d.actiu = 'S'
         GROUP BY TO_CHAR (D.DATADESPESA, 'YYYY');

      CURSOR C_E3 (p_anydespesa VARCHAR2) IS
         SELECT   IDPARLAMENTD, SUM (IMPORTTOTAL) AS SUMATOTAL
         FROM     DESPESA
         WHERE    ACTIU = 'S' AND TO_CHAR (DATADESPESA, 'YYYY') = P_ANYDESPESA
         GROUP BY IDPARLAMENTD;

      v_impmax        NUMBER := 0;
      v_idparlament   INTEGER;
   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E3';
      p_inlog := '';

      DELETE FROM e3;

      FOR V_ANY IN C_ANY LOOP
         FOR v_e3 IN c_e3 (v_any.ANYD) LOOP
            IF v_e3.SUMATOTAL > v_impmax THEN
               v_idparlament := v_e3.IDPARLAMENTD;
            END IF;
         END LOOP;

         INSERT INTO E3 (IDANY, RESULTAT)
         VALUES      (v_any.ANYD, v_idparlament);
      END LOOP;


      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E3[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modul_E3;
 /* **********************************************************************************************************
  *  NOM:         MODUL_E4
  *  DESCRIPCIO: ACTUALITZAR TAULA E4 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *              Calcula per cada  paralmentari la despesa total que ha tingu els darrers 4 anys 
  *  PARAMETERS:
  *  IN PUT ->  NO HI PARAMETRES D'ENTRADA PERQUE S RECALCULA TOTA LA TAULA.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/


   PROCEDURE modul_E4 (RSP OUT VARCHAR2) IS
      v_any      VARCHAR2 (4);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
      v_import   NUMBER;
       v_num       NUMBER;

      CURSOR c_E4 IS
         SELECT IDPARLAMENTARI FROM PARLAMENTARI;
   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E4';
      p_inlog := '';
      v_any := TO_CHAR (SYSDATE, 'YYYY');

      DELETE FROM E4;

      FOR V_E4 IN C_E4 LOOP
         SELECT NVL (SUM (IMPORTTOTAL), 0)
         INTO   v_import
         FROM   DESPESA
         WHERE  ACTIU = 'S' AND IDPARLAMENTARID = v_E4.IDPARLAMENTARI
         AND    TO_CHAR (DATADESPESA, 'YYYY') BETWEEN TO_CHAR (SYSDATE
                                                              ,'YYYY')
                                                      - 4
                                                  AND TO_CHAR (SYSDATE
                                                              ,'YYYY');

         INSERT INTO E4 (IDPARLAMENTARI, RESULTAT)
         VALUES      (v_E4.IDPARLAMENTARI, (trunc(v_import,2)));
      END LOOP;

      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E4[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modul_E4;

/* **********************************************************************************************************
  *  NOM:         MODUL_E5
  *  DESCRIPCIO: ACTUALITZAR TAULA E5 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *              Per cada any amb despeses asociades calcula la mitjana de despesa dun parlamentari durant aquell any 
  *              (tenint en compte tots els parlamentaris de tots els parlaments).
  *  PARAMETERS:
  *  IN PUT ->  NO HI PARAMETRES D'ENTRADA PERQUE S RECALCULA TOTA LA TAULA.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/


   PROCEDURE modul_E5 (RSP OUT VARCHAR2) IS
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
       v_num       NUMBER;

      CURSOR c_any IS
         SELECT   TO_CHAR (D.DATADESPESA, 'YYYY') AS ANYD
         FROM     despesa d
         WHERE    d.actiu = 'S' AND D.IDCATEGORIA = 1
         GROUP BY TO_CHAR (D.DATADESPESA, 'YYYY');

      v_mitja    NUMBER;
   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E5';
      p_inlog := '';

      DELETE FROM E5;

      FOR v_any IN c_any LOOP
         SELECT AVG (IMPORT_PARLAMENTARI)
         INTO   v_mitja
         FROM   (SELECT   PA.IDPARLAMENTARI
                         ,NVL (SUM (D.IMPORTTOTAL), 0) IMPORT_PARLAMENTARI
                 FROM     parlamentari pa, DESPESA D
                 WHERE    PA.IDPARLAMENTARI = D.IDPARLAMENTARID(+)
                 AND      NVL (D.ACTIU, 'S') = 'S'
                 AND      NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), v_any.ANYD) =
                             v_any.ANYD
                 GROUP BY IDPARLAMENTARI);

         INSERT INTO E5 (IDANY, RESULTAT)
         VALUES      (TO_NUMBER (v_any.ANYD), (trunc(v_mitja,2)));
      END LOOP;



      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E5[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         
   END modul_E5;

/* **********************************************************************************************************
  *  NOM:         MODUL_E6
  *  DESCRIPCIO: ACTUALITZAR TAULA E6 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *              Per cada any amb despeses asociades de parlamentaris calcula el parlamentari que ha tingut ms despesa associada
  *               i el que menys (tenint en compte tots els parlaments).
  *  PARAMETERS:
  *  IN PUT ->  NO HI PARAMETRES D'ENTRADA PERQUE S RECALCULA TOTA LA TAULA.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/



   PROCEDURE modul_E6 (RSP OUT VARCHAR2) IS
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
       v_num       NUMBER;

      CURSOR c_any IS
         SELECT   TO_CHAR (D.DATADESPESA, 'YYYY') AS ANYD
         FROM     despesa d
         WHERE    d.actiu = 'S' AND D.IDCATEGORIA = 1
         GROUP BY TO_CHAR (D.DATADESPESA, 'YYYY');

      CURSOR c_e6 (
         p_any VARCHAR2) IS
         SELECT   PA.IDPARLAMENTARI
                 ,NVL (SUM (D.IMPORTTOTAL), 0) AS IMPORTTOTAL
         FROM     despesa d, PARLAMENTARI PA
         WHERE    NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), p_any) = p_any
         AND      NVL (D.ACTIU, 'S') = 'S'
         AND      D.IDPARLAMENTARID(+) = PA.IDPARLAMENTARI
         GROUP BY IDPARLAMENTARI;

      v_impmax   NUMBER := 0;
      v_impmin   NUMBER := 0;
      v_idmax    INTEGER;
      v_idmin    INTEGER;
      v_primer   NUMBER;            --> indica si es el primer registre trobat
   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E3';
      p_inlog := '';

      DELETE FROM E6;

      FOR v_any IN c_any LOOP
         -- INICIALITZAM LES VARIABLES AUXILIARS
         v_primer := 1;
         v_impmax := 0;
         v_impmin := 0;
         v_idmin := NULL;
         v_idmax := NULL;

         FOR v_e6 IN c_e6 (v_any.ANYD) LOOP
            IF v_primer = 1 THEN
               v_impmin := v_e6.IMPORTTOTAL;
               v_idmin := v_e6.IDPARLAMENTARI;
               v_primer := 0;
            END IF;

            IF v_e6.IMPORTTOTAL >= v_impmax THEN
               v_impmax := v_e6.IMPORTTOTAL;
               v_idmax := v_e6.IDPARLAMENTARI;
            END IF;

            IF v_e6.IMPORTTOTAL <= v_impmin THEN
               v_impmin := v_e6.IMPORTTOTAL;
               v_idmin := v_e6.IDPARLAMENTARI;
            END IF;
         END LOOP;

         INSERT INTO E6 (IDANY, IDPARLAMENTARI0, IDPARLAMENTARI9)
         VALUES      (v_any.ANYD, v_idmin, v_idmax);
      END LOOP;



      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E6[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modul_E6;

 
 
/* **********************************************************************************************************
  *  NOM:         MODUL_E7
  *  DESCRIPCIO: ACTUALITZAR TAULA E7 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *              Per cada any amb despeses asociades  calcula  el nom del parlament amb 
  *              una mitjana de despesa associada als seus parlamentaris ms alta, 
  *              i el valor daquesta mitjana de despesa.
  *  PARAMETERS:
  *  IN PUT ->  NO HI PARAMETRES D'ENTRADA PERQUE S RECALCULA TOTA LA TAULA.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/

   PROCEDURE modul_E7 (RSP OUT VARCHAR2) IS
      p_nompro         VARCHAR2 (50);
      p_inlog          VARCHAR2 (500);
      p_outlog         VARCHAR2 (500);
      err_msg          VARCHAR2 (256);
      err_num          NUMBER;
      v_import         NUMBER;
      v_mitja          NUMBER;
      v_max            NUMBER := 0;
      v_idmax          INTEGER;
      v_nomparlament   PARLAMENT.NOM%TYPE;
       v_num       NUMBER;

      --v_any            VARCHAR2 (4);

      CURSOR c_any IS
         SELECT   TO_CHAR (D.DATADESPESA, 'YYYY') AS ANYD
         FROM     despesa d
         WHERE    d.actiu = 'S' AND D.IDCATEGORIA = 1
         GROUP BY TO_CHAR (D.DATADESPESA, 'YYYY');

      CURSOR c_e7 (
         p_any VARCHAR2) IS
         SELECT   D.IDPARLAMENTD
         FROM     despesa d
         WHERE    NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), P_any) = P_any
         AND      D.IDCATEGORIA = 1
         GROUP BY D.IDPARLAMENTD;
   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E7';
      p_inlog := ' ';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);

      --v_any := TO_CHAR (p_datadespesa, 'YYYY');
      DELETE FROM E7;

      FOR v_any IN c_any LOOP
         v_max := 0;
         v_idmax := NULL;

         FOR v_e7 IN c_e7 (v_any.ANYD) LOOP
            SELECT AVG (IMPORT_PARLAMENTARI)
            INTO   v_mitja
            FROM   (SELECT   PA.IDPARLAMENTARI
                            ,NVL (SUM (D.IMPORTTOTAL), 0) IMPORT_PARLAMENTARI
                    FROM     parlamentari pa, despesa d
                    WHERE    PA.IDPARLAMENTARI = D.IDPARLAMENTARID(+)
                    AND      PA.IDPARLAMENT = v_e7.IDPARLAMENTD
                    AND      NVL (TO_CHAR (D.DATADESPESA, 'YYYY'), v_any.ANYD) =
                                v_any.ANYD
                    AND      NVL (D.ACTIU, 'S') = 'S'
                    GROUP BY IDPARLAMENTARI);

            IF v_mitja > v_max THEN
               v_max := v_mitja;
               v_idmax := v_e7.IDPARLAMENTD;
            END IF;
         END LOOP;

         SELECT NOM
         INTO   v_nomparlament
         FROM   PARLAMENT
         WHERE  IDPARLAMENT = v_idmax;

         INSERT INTO E7 (IDANY, NOM, RESULTAT)
         VALUES      (v_any.ANYD, v_nomparlament, (trunc(v_max,2)));
      END LOOP;
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E7[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modul_E7;

/* **********************************************************************************************************
  *  NOM:         MODUL_E8
  *  DESCRIPCIO: ACTUALITZAR TAULA E8 PER MANTENIMENT EN TEMPS CONSTANT 1 DEL MODUL ESTADISTIC. 
  *              Per cada any amb despeses asociades calcula la suma de totes les despeses de tots els parlaments
  *              (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
  *  PARAMETERS:
  *  IN PUT ->  NO HI PARAMETRES D'ENTRADA PERQUE S RECALCULA TOTA LA TAULA.
  *  OUTPUT ->  RSP;
  *                   RETORNA:
  *                  'OK' SI L'EXECUCI S CORRECTA
  *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
  ***************************************************************************************************************/




   PROCEDURE modul_E8 (RSP OUT VARCHAR2) IS
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      err_msg    VARCHAR2 (256);
      err_num    NUMBER;
      v_import   NUMBER;


      CURSOR c_any IS
         SELECT   TO_CHAR (D.DATADESPESA, 'YYYY') AS ANYD
         FROM     despesa d
         WHERE    d.actiu = 'S'
         GROUP BY TO_CHAR (D.DATADESPESA, 'YYYY');

   BEGIN
      rsp := 'OK';
      p_nompro := 'modul_E8';
      p_inlog := '';

      DELETE FROM E8;

      FOR v_any IN c_any LOOP
         SELECT NVL (SUM (IMPORTTOTAL), 0)
            INTO   v_import
            FROM   DESPESA
            WHERE  ACTIU = 'S' 
            AND    TO_CHAR (DATADESPESA, 'YYYY') = v_any.ANYD;

            INSERT INTO E8 (IDANY, RESULTAT)
            VALUES      ( v_any.ANYD ,(trunc(v_import,2)));
      END LOOP;
      
     
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
   EXCEPTION
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modul_E8[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modul_E8;

END PAQ_EST;
/
