CREATE OR REPLACE PACKAGE                PAQ_PARLAMENT AS
/**************************************************************************
   Procediment per inserir una nova cambra paralemtaria
  ****************************************************************************/


PROCEDURE  altaParlament    
  ( p_tipusp              IN   PARLAMENT.TIPUSP%TYPE
   ,p_nom                IN   PARLAMENT.NOM%TYPE 
   ,p_adrea              IN   PARLAMENT.ADREA%TYPE
   ,p_telefon            IN   PARLAMENT.TELEFON%TYPE
   ,p_fax                IN   PARLAMENT.FAX%TYPE 
   ,p_web                IN   PARLAMENT.WEB%TYPE
   ,p_mail                IN   PARLAMENT.MAIL%TYPE 
   ,p_datacreacio         IN   PARLAMENT.DATACREACIO%TYPE,
   p_idpaisparlament     IN   PARLAMENT.IDPAISPARLAMENT%TYPE,
   p_inicilegislatura IN      PARLAMENT.INICILEGISLATURA%TYPE,
   p_finallegislatura IN      PARLAMENT.FINALLEGISLATURA%TYPE,
   p_idbanc             IN   PARLAMENT.IDBANC%TYPE,
   RSP               OUT       VARCHAR2);
  
   
  
  /****************************************************************************
     Funcio que retorna l'estat d'un Parlament 
     Si PARLAMENT.actiu ='S' . Ppodem realitzar operacions perqu el Parlament esta actiu
     Si PARLAMENT.actiu ='N'.  El Parlament s historic i no podem operar amb les seves dades.
    
  ****************************************************************************/
  
  
 FUNCTION estatParlament (p_idparlament IN INTEGER) RETURN VARCHAR2;
  
  
  
/****************************************************************************
  Procediment per donar de baixa una  cambre paralemtaria
  La baixa no s fsica, si no lgica.   Camvia l'atribut actiu = 'N' 
****************************************************************************/

PROCEDURE baixaParlament (  p_idparlament  IN     integer,
                            RSP            OUT VARCHAR2);


 
/*  modificacio Parlament */

PROCEDURE  modificacioParlament    
  (p_idparlament       IN   PARLAMENT.IDPARLAMENT%TYPE,
   p_tipusp               IN   PARLAMENT.TIPUSP%TYPE,
   p_nom                 IN   PARLAMENT.NOM%TYPE,
   p_adrea               IN   PARLAMENT.ADREA%TYPE,
   p_telefon             IN   PARLAMENT.TELEFON%TYPE,
   p_fax                 IN   PARLAMENT.FAX%TYPE,
   p_web                 IN   PARLAMENT.WEB%TYPE,
   p_mail                 IN   PARLAMENT.MAIL%TYPE, 
   p_datacreacio         IN   PARLAMENT.DATACREACIO%TYPE,
   p_idpaisparlament     IN   PARLAMENT.IDPAISPARLAMENT%TYPE,
   p_inicilegislatura  IN      PARLAMENT.INICILEGISLATURA%TYPE,
   p_finallegislatura  IN      PARLAMENT.FINALLEGISLATURA%TYPE,
   p_idbanc                IN   PARLAMENT.IDBANC%TYPE,
   RSP                 OUT       VARCHAR2);




END PAQ_PARLAMENT;
/


CREATE OR REPLACE PACKAGE BODY                                                          PAQ_PARLAMENT IS
   /***********************************************************************
   *  NOM:         ALTAPARLAMENT
   *  DESCRIPCIO:  GESTIONA L'ALTA D'UN PARLAMENT A LA TAULA PARLAMENT
   *               IDDPARLAMENT S AUTONUMERIC  I NO ES PASA PER PARAMETRES
   *               ACTIU S L'INDICADOR DEL ESTAT DEL REGISTRE A LA TAULA  I NO
   *               ES PASA PER PARAMETRES
   *               ACTIU = 'S' EL REGISTRE S VIGENT. PODEM OERAR AMB EL REGISTRE
   *               ACTIU = 'N' EL REGISTRE S HISTRIC. NO PODEM OPERAR AMB EL REGISTRE
   *               TOTALP = INDICA EL NUMERO DE PARLAMENTARIS DEL PARLAMENT.  
   *                        NO ES PASA PER PARAMENTRES. PER ALTA SEMPRE ES 0  
   *               CRIDA LA FUNCIO ALTALOG PER GRTAVAR EL RESULTAT DE L'EXECUCI.
   *  PARAMETERS:
   *  INPUT ->       P_TIPUSP ,P_NOM ,P_ADREA ,P_TELEFON ,P_FAX ,P_WEB
   *                ,P_MAIL ,P_DATACREACIO ,P_IDPAISPARLAMENT
   *                ,P_INICILEGISLATURA ,P_FINALLEGISLATURA ,P_IDBANC
   *  OUTPUT ->
   *             RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***********************************************************************/
   PROCEDURE altaParlament (
      p_tipusp             IN     PARLAMENT.TIPUSP%TYPE
     ,p_nom                IN     PARLAMENT.NOM%TYPE
     ,p_adrea            IN     PARLAMENT.ADREA%TYPE
     ,p_telefon            IN     PARLAMENT.TELEFON%TYPE
     ,p_fax                IN     PARLAMENT.FAX%TYPE
     ,p_web                IN     PARLAMENT.WEB%TYPE
     ,p_mail               IN     PARLAMENT.MAIL%TYPE
     ,p_datacreacio        IN     PARLAMENT.DATACREACIO%TYPE
     ,p_idpaisparlament    IN     PARLAMENT.IDPAISPARLAMENT%TYPE
     ,p_inicilegislatura   IN     PARLAMENT.INICILEGISLATURA%TYPE
     ,p_finallegislatura   IN     PARLAMENT.FINALLEGISLATURA%TYPE
     ,p_idbanc             IN     PARLAMENT.IDBANC%TYPE
     ,rsp                     OUT VARCHAR2) IS
      /*   declarem les variables */
      r_idparlament        INTEGER;
      r_tipusp             VARCHAR2 (30);
      r_nom                VARCHAR2 (60);
      r_totalp             INTEGER;
      r_adrea             VARCHAR2 (60);
      r_telefon            VARCHAR2 (20);
      r_fax                VARCHAR2 (20);
      r_web                VARCHAR2 (30);
      r_mail               VARCHAR2 (30);
      r_datacreacio        DATE;
      r_idpaisparlament    INTEGER;
      r_inicilegislatura   DATE;
      r_finallegislatura   DATE;
      r_idbanc             VARCHAR2 (60);
      r_actiu              VARCHAR2 (1);
      w_actiu              VARCHAR2 (1);

      p_nompro             VARCHAR2 (50);
      p_inlog              VARCHAR2 (500);
      p_outlog             VARCHAR2 (500);
      p_rsp                VARCHAR2 (350);

      err_msg              VARCHAR2 (256);
      err_num              NUMBER;
      wnulls               EXCEPTION;
      er_pais              EXCEPTION;
      er_dates             EXCEPTION;
      er_estad             EXCEPTION;
      notfnd               EXCEPTION;
      existe               EXCEPTION;
      w_modul              VARCHAR2 (2);
   BEGIN
      r_tipusp := SUBSTR (p_tipusp, 1, 30);
      r_nom := SUBSTR (p_nom, 1, 60);
      r_totalp := 0;
      r_adrea := SUBSTR (p_adrea, 1, 60);
      r_telefon := SUBSTR (p_telefon, 1, 20);
      r_fax := SUBSTR (p_fax, 1, 20);
      r_web := SUBSTR (p_web, 1, 30);
      r_mail := SUBSTR (p_mail, 1, 30);
      r_datacreacio := p_datacreacio;
      r_idpaisparlament := p_idpaisparlament;
      r_inicilegislatura := p_inicilegislatura;
      r_finallegislatura := p_finallegislatura;
      r_idbanc := SUBSTR (p_idbanc, 1, 60);
      r_actiu := 'S';

      --Emmagatzema els parmetres d'entrada per actualitzar el LOG
      p_nompro := 'alta parlament';
      p_inlog :=
            r_idparlament
         || ', '
         || p_tipusp
         || ', '
         || p_nom
         || ', '
         || r_totalp
         || ', '
         || p_adrea
         || ', '
         || p_telefon
         || ', '
         || p_fax
         || ', '
         || p_web
         || ', '
         || p_mail
         || ', '
         || p_datacreacio
         || ', '
         || p_idpaisparlament
         || ', '
         || p_inicilegislatura
         || ', '
         || p_finallegislatura
         || ', '
         || p_idbanc
         || ', '
         || r_actiu;

      --Controlar que els camps no siguin nuls
      IF (r_tipusp IS NULL)
         OR (r_nom IS NULL)
         OR (r_adrea IS NULL)
         OR (r_telefon IS NULL)
         OR (r_fax IS NULL)
         OR (r_web IS NULL)
         OR (r_mail IS NULL)
         OR (r_datacreacio IS NULL)
         OR (r_idpaisparlament IS NULL)
         OR (r_inicilegislatura IS NULL)
         OR (r_finallegislatura IS NULL)
         OR (r_idbanc IS NULL) THEN
         RAISE wnulls;
      END IF;
-- comprovar que les dates son correctes 
      IF (TRUNC (r_inicilegislatura) > TRUNC (SYSDATE)) THEN
         RAISE er_dates;
      END IF;

      IF TRUNC (r_inicilegislatura) >= TRUNC (r_finallegislatura) THEN
         RAISE er_dates;
      END IF;
      
--comprovar que el pais del parlament existeix

      w_actiu := PAQ_UTI.hiHaPais (p_idpaisparlament);

           
      IF w_actiu = 'S' then null; else
         RAISE er_pais;
      END IF;

      -- obtenir autonumeric per clau principal i donat l'alta del parlament ------
      SELECT S_PARLAMENT.NEXTVAL INTO r_idparlament FROM DUAL;

      INSERT INTO PARLAMENT (IDPARLAMENT
                            ,TIPUSP 
                            ,NOM
                            ,TOTALP
                            ,ADREA
                            ,TELEFON
                            ,FAX
                            ,WEB
                            ,MAIL
                            ,DATACREACIO
                            ,IDPAISPARLAMENT
                            ,INICILEGISLATURA
                            ,FINALLEGISLATURA
                            ,IDBANC
                            ,ACTIU)
      VALUES      (r_idparlament
                  ,r_tipusp
                  ,r_nom
                  ,r_totalp
                  ,r_adrea
                  ,r_telefon
                  ,r_fax
                  ,r_web
                  ,r_mail
                  ,r_datacreacio
                  ,r_idpaisparlament
                  ,r_inicilegislatura
                  ,r_finallegislatura
                  ,r_idbanc
                  ,r_actiu);

      -- ACTUALITZAR MODUL ESTADISTIC E1
      PAQ_EST.modul_E1 (RSP);
      w_modul := 'E1';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- alta PARLAMENT realitzada --');
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: Alta Parlament';
         w_actiu := NULL;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN existe THEN
         rsp := 'ERROR: Alta erronia. El Parlament ja existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_pais THEN
         rsp := 'ERROR: Alta erronia. El Pais no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Alta erronia. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp := 'ERROR: Alta erronia. Hi ha camps nulls';            
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_estad THEN
         rsp := 'ERROR: Modul ' || w_modul || ' Alta parlament';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modif.[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END altaParlament;


   /***********************************************************************
  *  NOM:         ESTATPARLAMENT
  *  DESCRIPCIO:  Serveix per saber si un parlament existeix i si esta actiu o  no
  *               CRIDA LA FUNCIO ALTALOG SI L'EXECUCI S ERRONIA.
  *  PARAMETERS:
  *  INPUT ->     P_IDPARLAMENT
  *  OUTPUT ->    P_ ACTIU
  *                   RETORNA:
  *                   Si  el parlament no existeix retorna NULLS a p_actiu
  *                   Si el parlament existeix retorna  el valor del camp ACTIU
  ***********************************************************************/
   FUNCTION estatParlament (p_idparlament INTEGER)
      RETURN VARCHAR2 IS
      RSP        VARCHAR2 (350);
      w_actiu    VARCHAR2 (1);
      wnulls     EXCEPTION;
      es_histo   EXCEPTION;
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      p_rsp      VARCHAR2 (350);
      err_msg    VARCHAR2 (50);
      err_num    NUMBER;
   BEGIN
   
-- preperar dades log   
      p_nompro := 'estatParlament';
      p_inlog := p_idparlament;
      
-- obtenir el valor del campo actiu
      SELECT ACTIU
      INTO   w_actiu
      FROM   PARLAMENT
      WHERE  (IDPARLAMENT = p_idparlament);

      IF w_actiu = 'N' THEN
         RAISE es_histo;
      END IF;

      p_outlog := w_actiu;
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: Parlament no existeix';
         w_actiu := NULL;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         RETURN null;
      WHEN es_histo THEN
         rsp := 'ERROR: El Parlament s Historic';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         RETURN null;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         RETURN null;
   END estatParlament;



   /***********************************************************************
   *  NOM:        BAIXAPARLAMENT
   *  DESCRIPCIO:  GESTIONA LA BAIXA D'UN PARLAMENT A LA TAULA PARLAMENT .
   *               LA BAIXA S LGICA PER PODER  MANTENIR TOTA LA INFORMACI DE  L'HISTORIC DE DESPESA
   *               CRIDA  A LA FUNCIO ESTATPARLAMENT.
   *               SI EL PARLAMENT EXISTEIX I ESTA ACTIU (ACTIU = 'S')PODEM REALITZAR LA BAIXA:
   *                   MODIFCAR PARLAMENT AMB ACTIU = 'N'
   *                   MODIFICAR TOTS ELS PARLAMENTARIS RELACIONATS AMB EL PARLAMENT AMB ACTIU = 'N'
   *               CRIDA LA FUNCIO ALTALOG PER GRAVAR EL RESULTAT DE L'EXECUCI.
   *  PARAMETERS:
   *  INPUT ->   P_IDPARLAMENT,
   *  OUTPUT ->  RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***********************************************************************/

   PROCEDURE baixaParlament (p_Idparlament IN INTEGER, rsp OUT VARCHAR2) IS
      w_actiu    VARCHAR2 (1);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      p_rsp      VARCHAR2 (350);
      err_msg    VARCHAR2 (50);
      err_num    NUMBER;
      es_histo   EXCEPTION;
      notfnd     EXCEPTION;
      wnulls     EXCEPTION;
      er_baixa   EXCEPTION;
      
-- definim un cursor per donar de baixa tots els parlamentaris del parlament
      CURSOR c_par IS
         SELECT *
         FROM   PARLAMENTARI
         WHERE  IDPARLAMENT = p_Idparlament AND ACTIU = 'S';
   BEGIN
      --S'emmagatzema els parmetres d'entrada per actualitzar el LOG
      p_nompro := 'baixa Parlament';
      p_inlog := p_idparlament || ', ' || w_actiu;
      p_outlog := ' ';

      IF p_idparlament IS NULL THEN
         RAISE wnulls;
      END IF;
--consultar si el parlament existeix i esta actiu
      w_actiu := estatParlament (p_idparlament);

      IF w_actiu IS NULL THEN
         RAISE notfnd;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo;
      END IF;

-- per donar de baixa el parlament modifiquem em camp actiu  a 'N'
      UPDATE PARLAMENT
      SET    ACTIU = 'N'
      WHERE  IDPARLAMENT = p_idparlament;

      /*
       Quan donem la baixa d'un parlament  tamb donem la baixa de tots
       els parlamentaris actius adscrits al parlament       */

      FOR v_par IN c_par LOOP
         PAQ_PARLAMENTARI.BAIXAPARLAMENTARI (v_par.IDPARLAMENTARI, p_rsp);

         IF p_rsp <> 'OK' THEN
            RAISE er_baixa;
         END IF;
      END LOOP;
      

      rsp := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- baixa PARLAMENT realitzada --');
      
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: BAIXA['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo THEN
         rsp := 'ERROR: Baixa erronia. El Parlament s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd THEN
         rsp := 'ERROR: Baixa erronia. El Parlament no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_baixa THEN
         rsp :=
            'ERROR: Baixa erronia. No s''han pogut donar de baixa els parlamentaris';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modif.[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END baixaParlament;


   /***********************************************************************
   *  NOM:         MODIFICACIOPARLAMENT
   *  DESCRIPCIO:  GESTIONA LA MODIFICACIO  D'UN PARLAMENT A LA TAULA PARLAMENT
   *               CRIDA  A LA FUNCIO ESTATPARLAMENT.
   *               SI EL PARLAMENT EXISTEIX I ACTIU = 'S'
   *               VALIDAR LES DADES I SI SN CORRECTES MODIFICAR EL  PARLAMENT.
   *               EL CAMP TOTALP NO ES POT MODIFICAR:    
   *                   TOTALP SE CALCULA DES DE ELS PROCEDIMENTS A/B/M PARLAMENTARI,
   *               CRIDA LA FUNCIO ALTALOG PER GRAVAR EL RESULTAT DE L'EXECUCI.
   *  PARAMETERS:
   *  INPUT ->   P_IDPARLAMENT
   *                 P_TIPUSP ,P_NOM ,P_ADREA ,P_TELEFON ,P_FAX ,P_WEB
   *                ,P_MAIL ,P_DATACREACIO ,P_IDPAISPARLAMENT
   *                ,P_INICILEGISLATURA ,P_FINALLEGISLATURA ,P_IDBANC
   *  OUTPUT ->  RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***********************************************************************/
   PROCEDURE modificacioParlament (
      p_idparlament        IN     PARLAMENT.IDPARLAMENT%TYPE
     ,p_tipusp             IN     PARLAMENT.TIPUSP%TYPE
     ,p_nom                IN     PARLAMENT.NOM%TYPE
     ,p_adrea            IN     PARLAMENT.ADREA%TYPE
     ,p_telefon            IN     PARLAMENT.TELEFON%TYPE
     ,p_fax                IN     PARLAMENT.FAX%TYPE
     ,p_web                IN     PARLAMENT.WEB%TYPE
     ,p_mail               IN     PARLAMENT.MAIL%TYPE
     ,p_datacreacio        IN     PARLAMENT.DATACREACIO%TYPE
     ,p_idpaisparlament    IN     PARLAMENT.IDPAISPARLAMENT%TYPE
     ,p_inicilegislatura   IN     PARLAMENT.INICILEGISLATURA%TYPE
     ,p_finallegislatura   IN     PARLAMENT.FINALLEGISLATURA%TYPE
     ,p_idbanc             IN     PARLAMENT.IDBANC%TYPE
     ,RSP                  OUT VARCHAR2) IS
     
      r_idparlament        INTEGER;
      r_tipusp             VARCHAR2 (30);
      r_nom                VARCHAR2 (60);
      r_adrea             VARCHAR2 (60);
      r_telefon            VARCHAR2 (20);
      r_fax                VARCHAR2 (20);
      r_web                VARCHAR2 (30);
      r_mail               VARCHAR2 (30);
      r_datacreacio        DATE;
      r_idpaisparlament    INTEGER;
      r_inicilegislatura   DATE;
      r_finallegislatura   DATE;
      r_idbanc             VARCHAR2 (60);
      w_actiu              VARCHAR2 (1);

      p_nompro             VARCHAR2 (50);
      p_inlog              VARCHAR2 (500);
      p_outlog             VARCHAR2 (500);
      p_rsp                VARCHAR2 (350);
      err_msg              VARCHAR2 (256);
      err_num              NUMBER;

      wnulls               EXCEPTION;
      notfnd1              EXCEPTION;
      notfnd2              EXCEPTION;
      es_histo             EXCEPTION;
      er_dates             EXCEPTION;
   BEGIN
      -- preparar les dades per LOG
      p_nompro := 'modificacio Parlament';
      p_inlog :=
            p_idparlament
         || ', '
         || p_tipusp
         || ', '
         || p_nom
         || ', '
         || p_adrea
         || ', '
         || p_telefon
         || ', '
         || p_fax
         || ', '
         || p_web
         || ', '
         || p_mail
         || ', '
         || p_datacreacio
         || ', '
         || p_idpaisparlament
         || ', '
         || p_inicilegislatura
         || ', '
         || p_finallegislatura
         || ', '
         || p_idbanc
         || ', '
         || w_actiu;


      --Controlar que els camps no siguin nuls
      IF (p_idparlament IS NULL)
         OR (p_tipusp IS NULL)
         OR (p_nom IS NULL)
         OR (p_adrea IS NULL)
         OR (p_telefon IS NULL)
         OR (p_fax IS NULL)
         OR (p_web IS NULL)
         OR (p_mail IS NULL)
         OR (p_datacreacio IS NULL)
         OR (p_idpaisparlament IS NULL)
         OR (p_inicilegislatura IS NULL)
         OR (p_finallegislatura IS NULL)
         OR (p_idbanc IS NULL) THEN
         RAISE wnulls;
      END IF;

      r_tipusp := SUBSTR (p_tipusp, 1, 30);
      r_nom := SUBSTR (p_nom, 1, 60);
      r_adrea := SUBSTR (p_adrea, 1, 60);
      r_telefon := SUBSTR (p_telefon, 1, 20);
      r_fax := SUBSTR (p_fax, 1, 20);
      r_web := SUBSTR (p_web, 1, 30);
      r_mail := SUBSTR (p_mail, 1, 30);
      r_datacreacio := p_datacreacio;
      r_idpaisparlament := p_idpaisparlament;
      r_inicilegislatura := p_inicilegislatura;
      r_finallegislatura := p_finallegislatura;
      r_idbanc := SUBSTR (p_idbanc, 1, 60);

      -- comprovar que el parlamentent existeix i esta actiu
      w_actiu := estatParlament (p_idparlament);

      IF w_actiu IS NULL THEN
         RAISE notfnd1;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo;
      END IF;

       -- comprovar que el pais del parlamenten es correcte
      w_actiu := PAQ_UTI.hiHaPais (p_idpaisparlament);

      IF w_actiu = 'N' THEN
         RAISE notfnd2;
      END IF;
    --comprovat que les dates son correctes
      IF (TRUNC (r_inicilegislatura) > TRUNC (SYSDATE)) THEN
         RAISE er_dates;
      END IF;

      IF TRUNC (r_inicilegislatura) >= TRUNC (r_finallegislatura) THEN
         RAISE er_dates;
      END IF;

      IF TRUNC (r_datacreacio) >= TRUNC (r_inicilegislatura) THEN
         RAISE er_dates;
      END IF;

      -- Si les dades son correctes modificar el parlament
      UPDATE PARLAMENT
      SET    TIPUSP = r_tipusp
            ,NOM = r_nom
            ,ADREA = r_adrea
            ,TELEFON = r_telefon
            ,FAX = r_fax
            ,WEB = r_web
            ,MAIL = r_mail
            ,DATACREACIO = r_datacreacio
            ,IDPAISPARLAMENT = r_idpaisparlament
            ,INICILEGISLATURA = r_inicilegislatura
            ,FINALLEGISLATURA = r_finallegislatura
            ,IDBANC = r_idbanc
      WHERE  idparlament = p_idparlament;

      rsp := 'OK';
      DBMS_OUTPUT.PUT_LINE ('-- Modificacio PARLAMENT realitzada --');
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: modif.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo THEN
         rsp := 'ERROR: Modificacio erronia. El Parlament s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Modificacio erronia. El Parlament no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd2 THEN
         rsp := 'ERROR: Modificacio erronia. El Pais no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_dates THEN
         rsp := 'ERROR: Alta erronia. El camps amb dates sn  erronis';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modif.[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modificacioParlament;
END PAQ_PARLAMENT;
/
