CREATE OR REPLACE PACKAGE                PAQ_PARLAMENTARI AS
/* procediment per donar d'alta un parlamentari  */
PROCEDURE  altaParlamentari   
   ( p_idpersona           IN   PARLAMENTARI.IDPERSONA%TYPE,
    p_dataalta            IN   PARLAMENTARI.DATAALTA%TYPE,
    p_adrea              IN   PARLAMENTARI.ADREA%TYPE, 
    p_telefon             IN   PARLAMENTARI.TELEFON%TYPE,
    p_fax                 IN   PARLAMENTARI.FAX%TYPE,
    p_mail                IN   PARLAMENTARI.MAIL%TYPE,
    p_idvisa              IN   PARLAMENTARI.IDVISA%TYPE,
    p_idpartit            IN   PARLAMENTARI.IDPARTIT%TYPE,
    p_idcarrec            IN   PARLAMENTARI.IDCARREC%TYPE,
    p_idparlament         IN   PARLAMENTARI.IDPARLAMENT%TYPE,
     RSP                  OUT       VARCHAR2);
  
  /****************************************************************************
     Funcio que retorna l'estat d'un Parlamentari 
     Si PARLAMENT.actiu ='S'  podem realitzar operacions perqu el Parlamentari esta actiu
     Si PARLAMENT.actiu ='N' . El Parlament s historic i no podem operar amb les seves dades.
    ****************************************************************************/
     
  
  FUNCTION estatParlamentari (p_idparlamentari IN INTEGER) RETURN VARCHAR2;
  
   
/****************************************************************************
  Procediment per donar de baixa un  parlamentari 
  La baixa no s fsica, si no lgica. (  Camvia  l'atribut actiu = 'N' )
****************************************************************************/

PROCEDURE baixaParlamentari ( 
  p_idparlamentari      IN     integer,
  RSP                OUT VARCHAR2);


 
/*  Procediment per  modificacio Parlamentari */

PROCEDURE  modificacioParlamentari    
  ( p_idparlamentari      IN   PARLAMENTARI.IDPARLAMENTARI%TYPE,
    p_idpersona           IN   PARLAMENTARI.IDPERSONA%TYPE,
    p_dataalta            IN   PARLAMENTARI.DATAALTA%TYPE,
    p_adrea              IN   PARLAMENTARI.ADREA%TYPE,
    p_telefon             IN   PARLAMENTARI.TELEFON%TYPE,
    p_fax                 IN   PARLAMENTARI.FAX%TYPE,
    p_mail                IN   PARLAMENTARI.MAIL%TYPE,
    p_idvisa              IN   PARLAMENTARI.IDVISA%TYPE,
    p_idpartit            IN   PARLAMENTARI.IDPARTIT%TYPE,
    p_idcarrec            IN   PARLAMENTARI.IDCARREC%TYPE,
    RSP                  OUT       VARCHAR2);

 
 
END PAQ_PARLAMENTARI;
/


CREATE OR REPLACE PACKAGE BODY                                                                                             PAQ_PARLAMENTARI AS
   /***********************************************************************
   *  NOM:         ALTAPARLAMENTARI
   *  DESCRIPCIO:  GESTIONA L'ALTA D'UN PARLAMENTARI A LA TAULA PARLAMENTARI
   *               IDDPARLAMENTARI S AUTONUMERIC  I NO ES PASA PER PARAMETRES
   *               ACTIU S L'INDICADOR DEL ESTAT DEL REGISTRE A LA TAULA  I NO
   *               ES PASA PER PARAMETRES
   *               ACTIU SE CREA A 'S'
   *               ACTIU = 'S' EL REGISTRE S VIGENT. PODEM OERAR AMB EL REGISTRE
   *               ACTIU = 'N' EL REGISTRE S HISTRIC. NO PODEM OPERAR AMB EL REGISTRE
   *               CADA ALTA DE PARLAMENTARI  GENERA UNA MODIFICACI A LA TAULA
   *               PARLAMENT PER ACTUALITZAR EL CAMP TOTALP.
   *               CRIDA LA FUNCIO ALTALOG PER GRTAVAR EL RESULTAT DE L'EXECUCI.
   *  PARAMETERS:
   *  INPUT ->
   *                P_IDPERSONA ,P_DATAALTA ,P_ADREA ,P_TELEFON ,P_FAX ,P_MAIL
   *              ,P_IDVISA ,P_IDPARTIT ,P_IDCARREC ,P_IDPARLAMENT
   *  OUTPUT ->
   *             RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***********************************************************************/
   PROCEDURE altaParlamentari (
      p_idpersona     IN     PARLAMENTARI.IDPERSONA%TYPE
     ,p_dataalta      IN     PARLAMENTARI.DATAALTA%TYPE
     ,p_adrea       IN     PARLAMENTARI.ADREA%TYPE
     ,p_telefon       IN     PARLAMENTARI.TELEFON%TYPE
     ,p_fax           IN     PARLAMENTARI.FAX%TYPE
     ,p_mail          IN     PARLAMENTARI.MAIL%TYPE
     ,p_idvisa        IN     PARLAMENTARI.IDVISA%TYPE
     ,p_idpartit      IN     PARLAMENTARI.IDPARTIT%TYPE
     ,p_idcarrec      IN     PARLAMENTARI.IDCARREC%TYPE
     ,p_idparlament   IN     PARLAMENTARI.IDPARLAMENT%TYPE
     ,RSP                OUT VARCHAR2) IS
      r_idparlamentari   INTEGER;
      r_idpersona        INTEGER;
      r_dataalta         DATE;
      r_adrea          VARCHAR2 (60);
      r_telefon          VARCHAR2 (10);
      r_fax              VARCHAR2 (10);
      r_mail             VARCHAR2 (25);
      r_idvisa           VARCHAR2 (25);
      r_idpartit         INTEGER;
      r_idcarrec         INTEGER;
      r_idparlament      INTEGER;
      r_actiu            VARCHAR2 (1);


      w_actiu            VARCHAR2 (1);
      p_nompro           VARCHAR2 (50);
      p_inlog            VARCHAR2 (500);
      p_outlog           VARCHAR2 (500);

      err_msg            VARCHAR2 (256);
      err_num            NUMBER;
      w_modul            VARCHAR2 (2);

      wnulls             EXCEPTION;
      es_histo1          EXCEPTION;
      es_histo2          EXCEPTION;
      notfnd1            EXCEPTION;
      notfnd2            EXCEPTION;
      notfnd3            EXCEPTION;
      notfnd4            EXCEPTION;
      er_estad           EXCEPTION;
   BEGIN
      r_idpersona := p_idpersona;
      r_dataalta := p_dataalta;
      r_adrea := SUBSTR (p_adrea, 1, 60);
      r_telefon := SUBSTR (p_telefon, 1, 10);
      r_fax := SUBSTR (p_fax, 1, 10);
      r_mail := SUBSTR (p_mail, 1, 25);
      r_idvisa := SUBSTR (p_idvisa, 1, 25);
      r_idpartit := p_idpartit;
      r_idcarrec := p_idcarrec;
      r_idparlament := p_idparlament;
      r_actiu := 'S';

      --S'emmagatzema els parmetres d'entrada per actualitzar el LOG
      p_nompro := 'alta parlamentari';
      p_inlog :=
            r_idparlamentari
         || ',    '
         || p_idpersona
         || ',    '
         || p_dataalta
         || ',    '
         || p_adrea
         || ',    '
         || p_telefon
         || ',    '
         || p_fax
         || ',    '
         || p_mail
         || ',    '
         || p_idvisa
         || ',    '
         || p_idpartit
         || ',    '
         || p_idcarrec
         || ',    '
         || p_idparlament
         || ',    '
         || r_actiu;

      --Controlar que els camps no siguin nuls
      IF (r_idpersona IS NULL)
         OR (r_dataalta IS NULL)
         OR (r_adrea IS NULL)
         OR (r_telefon IS NULL)
         OR (r_fax IS NULL)
         OR (r_mail IS NULL)
         OR (r_idvisa IS NULL)
         OR (r_idpartit IS NULL)
         OR (r_idcarrec IS NULL)
         OR (r_idparlament IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- VALIDAR QUE EL CODI DE PERSONA SIGUI CORRECTE

      w_actiu := PAQ_UTI.hiHaPersona (p_idpersona);

      IF w_actiu = 'N' THEN
         RAISE notfnd1;
      END IF;
         
   --VALIDAR QUE EL PARTIT DEL PARLAMENTARI ES CORRECTE
      w_actiu := PAQ_UTI.hiHaPartit (p_idpartit);

      IF w_actiu = 'N' THEN
         RAISE notfnd3;
      END IF;
  -- VALIDAR QUE EL CARREC QUE OCUPA EL PARLAMENTARI S CORRECTE
      w_actiu := PAQ_UTI.hiHaCarrec (p_idcarrec);

      IF w_actiu = 'N' THEN
         RAISE notfnd4;
      END IF;
 -- comprovar que el parlamentent existeix i esta actiu
      w_actiu := paq_parlament.estatParlament (p_idparlament);
      IF w_actiu = 'S' THEN null; else
         RAISE notfnd2;
      END IF;
     
      -- OBTENIR EL VALOR AUTONUMERIC DE LA CLAU IDPARLAMENTARI
      SELECT S_PARLAMENTARI.NEXTVAL INTO r_idparlamentari FROM DUAL;

      INSERT INTO PARLAMENTARI (Idparlamentari
                               ,idpersona
                               ,dataalta
                               ,adrea
                               ,telefon
                               ,fax
                               ,mail
                               ,idvisa
                               ,idpartit
                               ,idcarrec
                               ,idparlament
                               ,actiu)
      VALUES      (r_idparlamentari
                  ,r_idpersona
                  ,r_dataalta
                  ,r_adrea
                  ,r_telefon
                  ,r_fax
                  ,r_mail
                  ,r_idvisa
                  ,r_idpartit
                  ,r_idcarrec
                  ,r_idparlament
                  ,r_actiu);

      -- ACTUALITZAR MODUL ESTADISTIC E2
      PAQ_EST.modul_E2 (r_idparlament, RSP);
      w_modul := 'E2';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;
      
      -- ACTUALITZAR MODUL ESTADISTIC E4
      PAQ_EST.modul_E4 (RSP);
      w_modul := 'E4';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;
      
      -- ACTUALITZAR MODUL ESTADISTIC E5
      PAQ_EST.modul_E5 ( RSP);
      w_modul := 'E5';
      
      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;
    
      -- ACTUALITZAR MODUL ESTADISTIC E6
      PAQ_EST.modul_E6 (RSP);
      w_modul := 'E6';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;
      
      -- ACTUALITZAR MODUL ESTADISTIC E7
      PAQ_EST.modul_E7 (RSP);
      w_modul := 'E7';

      IF RSP <> 'OK' THEN
         RAISE er_estad;
      END IF;

      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
  -- MODIFICAR EL CONTADOR DE PARLAMENTARIS DE LA TAULA PARLAMENT
      UPDATE parlament
      SET    totalp = totalp + 1
      WHERE  idparlament = r_idparlament;

      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- alta PARLAMENTARI realitzada --');
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Alta erronia. La Persona no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd2 THEN
         rsp := 'ERROR: Alta erronia. El Parlament no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo2 THEN
         rsp := 'ERROR: Alta erronia. El Parlament s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd3 THEN
         rsp := 'ERROR: Alta erronia. El Partit no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd4 THEN
         rsp := 'ERROR: Alta erronia. El Carrec no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN er_estad THEN
         rsp := 'ERROR: Modul ' || w_modul || ' Alta parlamentari';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         rsp := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END altaParlamentari;

   /***********************************************************************
   *  NOM:         ESTATPARLAMENTARI
   *  DESCRIPCIO:  SERVEIX PER SABER SI UN PARLAMENTARI EXISTEIX I SI ESTA ACTIU O  NO
   *                      CRIDA LA FUNCIO ALTALOG SI L'EXECUCI S ERRONIA
   *  PARAMETERS:
   *  INPUT ->     P_IDPARLAMENTARI
   *  OUTPUT ->    P_ ACTIU
   *                   RETORNA:
   *                   SI  EL PARLAMENT NO EXISTEIX RETORNA NULLS A P_ACTIU
   *                   SI EL PARLAMENT EXISTEIX RETORNA  EL VALOR DEL CAMP ACTIU
   ***********************************************************************/

   FUNCTION estatParlamentari (p_idparlamentari IN INTEGER)
      RETURN VARCHAR2 IS
      RSP        VARCHAR2 (50);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);

      err_msg    VARCHAR2 (50);
      err_num    NUMBER;
      w_actiu    VARCHAR2 (1);
   BEGIN
   
     --S'emmagatzemen  els parmetres d'entrada per actualitzar el LOG
      p_nompro := 'estatParlamentari';
      p_inlog := p_idparlamentari;
      w_actiu := 'N';
    -- comprovar l'estat del parlamentari
      SELECT ACTIU
      INTO   w_actiu
      FROM   PARLAMENTARI
      WHERE  (IDPARLAMENTARI = p_idparlamentari);

      p_outlog := w_actiu;
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);

      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: Parlamentari no existeix';
         w_actiu := NULL;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         return null;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
         return null;
   END estatParlamentari;


   /***********************************************************************
   *  NOM:        BAIXAPARLAMENTARI
   *  DESCRIPCIO:  GESTIONA LA BAIXA D'UN PARLAMENTARI  LA TAULA PARLAMENTARI .
   *                LA BAIXA S LGICA. PER PODER  MANTENIR TOTA LA INFORMACI DE  L'HISTORIC DE DESPESA
   *               CRIDA  A LA FUNCIO ESTATPARLAMENTIARI
   *              SI EL PARLAMENTARI EXISTEIX I ACTIU = 'S'
   *                 MODIFIQUEM PARLAMENTARI AMB ACTIU = 'N'
   *                MODIFIQUEM EL CAMP TOTALP DE PARLAMENT   AMB TOTS ELS PARLAMENTARIS RELACIONATS AMB EL PARLAMENT
   *               CRIDA LA FUNCIO ALTALOG PER GRTAVAR EL RESULTAT DE L'EXECUCI.
   *  PARAMETERS:
   *  INPUT ->   P_IDPARLAMENTARI,
   *  OUTPUT ->
   *             RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***********************************************************************/



   PROCEDURE baixaParlamentari (p_Idparlamentari   IN     INTEGER
                               ,rsp                   OUT VARCHAR2) IS
      w_actiu         VARCHAR2 (1);
      p_nompro        VARCHAR2 (50);
      p_inlog         VARCHAR2 (500);
      p_outlog        VARCHAR2 (500);

      err_msg         VARCHAR2 (50);
      err_num         NUMBER;
      w_idparlament   INTEGER;

      wnulls          EXCEPTION;
      es_histo        EXCEPTION;
      notfnd          EXCEPTION;
   BEGIN
      --S'emmagatzema els parmetres d'entrada per actualitzar el LOG
      p_nompro := 'baixa Parlamentari';
      p_inlog := p_idparlamentari || ', ' || w_actiu;
      p_outlog := ' ';
      
      -- VALIDAR SI EL PARLAMENTARI A MODIFICAR S CORRECTE
      IF p_idparlamentari IS NULL THEN
         RAISE wnulls;
      END IF;
      -- comprovar si el parlamentari existeix
      w_actiu := estatParlamentari (p_idparlamentari);


      IF w_actiu IS NULL THEN
         RAISE notfnd;
      END IF;
      IF w_actiu = 'N' THEN
         RAISE es_histo;
      END IF;

  -- baixa lgica del parlmentari. Canviar actiu a 'N'
      UPDATE PARLAMENTARI
      SET    ACTIU = 'N'
      WHERE  IDPARLAMENTARI = p_idparlamentari;

      SELECT idparlament
      INTO   w_idparlament
      FROM   PARLAMENTARI
      WHERE  (IDPARLAMENTARI = p_idparlamentari);

      -- MODIFICAR EL COMPTADOR DE PARLAMENTARIS DEL PARLAMENT
      UPDATE parlament
      SET    totalp = totalp - 1
      WHERE  idparlament = w_idparlament;

      rsp := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- baixa PARLAMENTARI realitzada --');
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: baixa.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd THEN
         rsp := 'ERROR: Baixa erronia. El Parlamentari no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo THEN
         rsp := 'ERROR: Baixa erronia. El Parlamentari s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR:baixa.[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END baixaParlamentari;


   /***********************************************************************
   *  NOM:         MODIFICACIOPARLAMENTARI
   *  DESCRIPCIO:  GESTIONA LA MODIFICACIO  D'UN PARLAMENTARI  A LA TAULA PARLAMENTARI
   *               CRIDA  A LA FUNCIO ESTATPARLAMENTARI.
   *              SI EL PARLAMENTARI  EXISTEIX I ACTIU = 'S'
   *                   VALIDAR LES DADES I SI SN CORRECTES MODIFICAR EL  PARLAMENTARI
   *              EL PARLAMENTARI NO POT CANVIAR DE PARLAMENT. NO ES PERMET MODIFICAR AQUESTA DADA.
   *              CRIDA LA FUNCIO ALTALOG PER GRAVAR EL RESULTAT DE L'EXECUCI.
   *  PARAMETERS:
   *  INPUT ->   P_IDPARLAMENTARI
   *                P_IDPERSONA ,P_DATAALTA ,P_ADREA ,P_TELEFON ,P_FAX ,P_MAIL
   *              ,P_IDVISA ,P_IDPARTIT ,P_IDCARREC
   *  OUTPUT ->
   *             RSP ;
   *                   RETORNA:
   *                  'OK' SI L'EXECUCI S CORRECTA
   *                  'ERROR:' + 'TIPUS D'ERROR' SI L'EXECUCI S ERRONEA'
   ***********************************************************************/

   PROCEDURE modificacioParlamentari (
      p_idparlamentari   IN     PARLAMENTARI.IDPARLAMENTARI%TYPE
     ,p_idpersona        IN     PARLAMENTARI.IDPERSONA%TYPE
     ,p_dataalta         IN     PARLAMENTARI.DATAALTA%TYPE
     ,p_adrea          IN     PARLAMENTARI.ADREA%TYPE
     ,p_telefon          IN     PARLAMENTARI.TELEFON%TYPE
     ,p_fax              IN     PARLAMENTARI.FAX%TYPE
     ,p_mail             IN     PARLAMENTARI.MAIL%TYPE
     ,p_idvisa           IN     PARLAMENTARI.IDVISA%TYPE
     ,p_idpartit         IN     PARLAMENTARI.IDPARTIT%TYPE
     ,p_idcarrec         IN     PARLAMENTARI.IDCARREC%TYPE
     ,RSP                   OUT VARCHAR2) IS
      r_idparlamentari   INTEGER;
      r_idpersona        INTEGER;
      r_dataalta         DATE;
      r_adrea          VARCHAR2 (60);
      r_telefon          VARCHAR2 (10);
      r_fax              VARCHAR2 (10);
      r_mail             VARCHAR2 (25);
      r_idvisa           VARCHAR2 (25);
      r_idpartit         INTEGER;
      r_idcarrec         INTEGER;


      w_actiu            VARCHAR2 (1);
      p_nompro           VARCHAR2 (50);
      p_inlog            VARCHAR2 (500);
      p_outlog           VARCHAR2 (500);
      err_msg            VARCHAR2 (256);
      err_num            NUMBER;

      wnulls             EXCEPTION;
      es_histo1          EXCEPTION;
      es_histo2          EXCEPTION;
      notfnd1            EXCEPTION;
      notfnd2            EXCEPTION;
      notfnd3            EXCEPTION;
      p_nompro := 'modificacio Parlamentari';
      notfnd4            EXCEPTION;
   BEGIN
      -- PERPARAR LES DADES PER  ACTUALITZAR LA TAULA LOG
      p_inlog :=
            p_idparlamentari
         || ',    '
         || p_idpersona
         || ',    '
         || p_dataalta
         || ',    '
         || p_adrea
         || ',    '
         || p_telefon
         || ',    '
         || p_fax
         || ',    '
         || p_mail
         || ',    '
         || p_idvisa
         || ',    '
         || p_idpartit
         || ',    '
         || p_idcarrec;


      r_idparlamentari := p_idparlamentari;
      r_idpersona := p_idpersona;
      r_dataalta := p_dataalta;
      r_adrea := SUBSTR (p_adrea, 1, 60);
      r_telefon := SUBSTR (p_telefon, 1, 10);
      r_fax := SUBSTR (p_fax, 1, 10);
      r_mail := SUBSTR (p_mail, 1, 25);
      r_idvisa := SUBSTR (p_idvisa, 1, 25);
      r_idpartit := p_idpartit;
      r_idcarrec := p_idcarrec;

      -- VALIDAR SI HI HA DADES AMB NULLS
      IF (r_idpersona IS NULL)
         OR (r_dataalta IS NULL)
         OR (r_adrea IS NULL)
         OR (r_telefon IS NULL)
         OR (r_fax IS NULL)
         OR (r_mail IS NULL)
         OR (r_idvisa IS NULL)
         OR (r_idpartit IS NULL)
         OR (r_idcarrec IS NULL) THEN
         RAISE wnulls;
      END IF;

      -- VALIDA L'ESTAT  DEL PARLAMENTARI
      w_actiu := estatParlamentari (p_idparlamentari);

    IF w_actiu IS NULL THEN
         RAISE notfnd1;
      END IF;

      IF w_actiu = 'N' THEN
         RAISE es_histo1;
      END IF;
      
      
    -- VALIDA QUE EL CODI DE PERSONA SIGUI CORRECTE
      w_actiu := PAQ_UTI.hiHaPersona (p_idpersona);

      IF w_actiu = 'N' THEN
         RAISE notfnd2;
      END IF;
    -- VALIDA QUE EL CODI DE PARTIT SIGUI CORRECTE
      w_actiu := PAQ_UTI.hiHaPartit (p_idpartit);

      IF w_actiu = 'N' THEN
         RAISE notfnd3;
      END IF;
    -- VALIDA QUE EL CODI DE CARREC SIGUI CORRECTE
      w_actiu := PAQ_UTI.hiHaCarrec (p_idcarrec);

      IF w_actiu = 'S' THEN null; else
         RAISE notfnd4;
      END IF;

      -- Modificacio  registe PARLAMENTARI

      UPDATE PARLAMENTARI
      SET    idpersona = p_idpersona
            ,dataalta = p_dataalta
            ,adrea = p_adrea
            ,telefon = p_telefon
            ,fax = p_fax
            ,mail = p_mail
            ,idvisa = p_idvisa
            ,idpartit = p_idpartit
            ,idcarrec = p_idcarrec
      WHERE  idparlamentari = p_idparlamentari;


      p_outlog := 'OK';
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);
      COMMIT;
      DBMS_OUTPUT.PUT_LINE ('-- Modificacio PARLAMENTARI realitzada --');
   EXCEPTION
      WHEN wnulls THEN
         err_num := SQLCODE;
         rsp :=
               'ERROR: modif.['
            || TO_CHAR (err_num)
            || '] '
            || ' Hi ha camps nuls';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd1 THEN
         rsp := 'ERROR: Modificacio erronia. El Parlamentari no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN es_histo1 THEN
         rsp := 'ERROR: Modificacio erronia. El Parlamentari s Histric';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd3 THEN
         rsp := 'ERROR: Modificacio erronia. El Partit no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN notfnd4 THEN
         rsp := 'ERROR: Modificacio erronia. El Carrec no existeix';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: modif.[' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   END modificacioParlamentari;
END PAQ_PARLAMENTARI;
/
