CREATE OR REPLACE PACKAGE                       PAQ_UTI AS
   /*  gestiona l'alta d'un registre a la taula LOG*/
   PROCEDURE ALTALOG (p_nompro   IN VARCHAR2
                     ,p_inlog    IN VARCHAR2
                     ,p_outlog   IN VARCHAR2
                     ,p_rsp      IN VARCHAR2);

   /* funcio que retorna l'estat d'un registre de la taula PAIS */
   FUNCTION hiHaPais (p_idpais IN INTEGER)
      RETURN VARCHAR2;

   /* funcio que retorna l'estat d'un registre de la taula PERSONA */
   FUNCTION hiHaPersona (p_idpersona IN INTEGER)
      RETURN VARCHAR2;

   /* funcio que retorna l'estat d'un registre de la taula PARTIT */
   FUNCTION hiHaPartit (p_idpartit IN INTEGER)
      RETURN VARCHAR2;

   /* funcio que retorna l'estat d'un registre de la taula CARREC */
   FUNCTION hiHaCarrec (p_idcarrec IN INTEGER)
      RETURN VARCHAR2;

   /* funcio que retorna l'estat d'un registre de la taula PROVEIDOR */
   FUNCTION hiHaproveidor (p_idproveidor IN INTEGER)
      RETURN VARCHAR2;

   /* funcio que retorna si la codificacio de la categoria de la depesa s correcta*/
   FUNCTION hiHaCategoria (p_idCategoria IN INTEGER)
      RETURN VARCHAR2;

   /* funcio que retorna l'estat d'un registre de la taula TIPUS_DESPESA */
   FUNCTION hiHaTipusDespesa (p_idtipus IN INTEGER)
      RETURN VARCHAR2;

   /* funcio que retorna l'estat d'un registre de la taula ESTAT */
   FUNCTION hiHaEstatDespesa (p_idestat IN INTEGER)
      RETURN VARCHAR2;
   /* funcio que retorna si la codificaci del format any s correcte */
   FUNCTION AnyOK (p_any VARCHAR2)
      RETURN VARCHAR2;
   /* funcio que retorna l'indicador per actualitzar o no el mdul estadstic */
   FUNCTION calcularEstadistica (p_datadespesa    DATE
                                ,p_importbrut     NUMBER
                                ,p_tx100iva       NUMBER
                                ,p_iddespesa      INTEGER)
      RETURN VARCHAR2;
END PAQ_UTI;
/


CREATE OR REPLACE PACKAGE BODY                                                          PAQ_UTI IS
   /******************************************************************************
       NOM:         ALTALOG
       PROPSIT:    GESTIONA L'ALTA D'UN REGISTRE A LA TAULA LOG
       PARAMETERS:  
       INPUT:       P_NOMPRO, DATAHORA, P_INLOG, P_POUTLOG
       OUTPUT:      RST
       RETORNA:     'OK' SI HA ANAT B, 'ERROR:' +  'TIPUS D'ERROR' SI NO HA ANAT B
       ******************************************************************************/

   PROCEDURE ALTALOG (p_nompro   IN VARCHAR2
                     ,p_inlog    IN VARCHAR2
                     ,p_outlog   IN VARCHAR2
                     ,p_rsp      IN VARCHAR2) IS
      p_idLog   INTEGER;

      rsp       VARCHAR2 (350); 
      err_msg   VARCHAR2 (256);
      err_num   NUMBER;
      wnulls    EXCEPTION;
   BEGIN
      -- OBTENIR EL VALOR AUTONUMERIC PER CODIFICAR EL CAMP IDLOG
      SELECT S_LOGT.NEXTVAL INTO p_idLog FROM DUAL;

      -- inserim  el registre
      INSERT INTO LOG (idlog
                      ,nompro
                      ,datahora
                      ,pinlog
                      ,poutlog
                      ,rsp)
      VALUES      (p_idlog
                  ,p_nompro
                  ,SYSDATE
                  ,p_inlog
                  ,p_outlog
                  ,p_rsp);
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR LOG : NO S''HA POGUT OBTENIR LA CLAU DEL LOG';
         DBMS_OUTPUT.PUT_LINE (rsp);
      WHEN OTHERS THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 256);
         rsp := 'ERROR LOG: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
   END ALTALOG;

   /******************************************************************************
       NOM:         HIHAPAIS
       PROPSIT:    Retorna l'estat d'un registre de la taula PAIS
       PARAMETERS:
       INPUT:       P_IDPAIS
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CLAU NO EXISTEIX.
                    'S'  SI LA CLAU EXISTEIX
       ******************************************************************************/
   FUNCTION hiHaPais (p_idpais IN INTEGER)
      RETURN VARCHAR2 IS
      RSP        VARCHAR2 (50);
      w_actiu    VARCHAR2 (1);
      p_nompro   VARCHAR2 (50);
      p_inlog    VARCHAR2 (500);
      p_outlog   VARCHAR2 (500);
      p_rsp      VARCHAR2 (350);
      err_msg    VARCHAR2 (50);
      err_num    NUMBER;
      w_idpais   INTEGER;
   BEGIN
-- preparar dades per taula LOG
      p_nompro := 'Hi Ha  Pais';
      p_inlog := p_idpais;
      w_actiu := 'N';
-- accedir a la taula pais per comprovar si el registre existeix      
      SELECT idpais
      INTO   w_idpais
      FROM   PAIS
      WHERE  (idpais = p_idpais);
-- si no sur per error el registre existeix
      p_outlog := p_idpais;
      RSP := 'OK';
      w_actiu := 'S';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,RSP);
      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: NO Hi Ha Pais';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
         RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
         RETURN w_actiu;
   END hiHaPais;

   /******************************************************************************
       NOM:         HIHAPERSONA
       PROPSIT:    Retorna l'estat d'un registre de la taula PERSONA
       PARAMETERS:
       INPUT:       P_IDPERSONA
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CLAU NO EXISTEIX.
                    'S'  SI LA CLAU EXISTEIX
       ******************************************************************************/
   ---------------------------------------------------------
   FUNCTION hiHaPersona (p_idpersona IN INTEGER)
      RETURN VARCHAR2 IS
      RSP           VARCHAR2 (50);
      w_actiu       VARCHAR2 (1);
      p_nompro      VARCHAR2 (50);
      p_inlog       VARCHAR2 (100);
      p_outlog      VARCHAR2 (100);
      p_rsp         VARCHAR2 (50);
      err_msg       VARCHAR2 (50);
      err_num       NUMBER;
      w_idpersona   INTEGER;
   BEGIN
   -- preparar les  dades per taula LOG
      p_nompro := 'Hi Ha  Persona';
      p_inlog := p_idpersona;
      w_actiu := 'N';
  -- consulta a la taula persona si la clau existeix 
      SELECT idpersona
      INTO   w_idpersona
      FROM   PERSONA
      WHERE  (idpersona = p_idpersona);
   -- si no surt per error el registe existeix
      p_outlog := w_idpersona;
      RSP := 'OK';
      w_actiu := 'S';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,RSP);
      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: NO Hi Ha  Persona';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
         RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
        RETURN w_actiu;
   END hiHaPersona;


   /******************************************************************************
       NOM:         HIHAPARTIT
       PROPSIT:    Retorna l'estat d'un registre de la taula PARTIT
       PARAMETERS:
       INPUT:       P_IDPARTIT
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CLAU NO EXISTEIX.
                    'S'  SI LA CLAU EXISTEIX
       ******************************************************************************/
   FUNCTION hiHaPartit (p_idpartit IN INTEGER)
      RETURN VARCHAR2 IS
      RSP          VARCHAR2 (50);
      w_actiu      VARCHAR2 (1);
      p_nompro     VARCHAR2 (50);
      p_inlog      VARCHAR2 (100);
      p_outlog     VARCHAR2 (100);
      p_rsp        VARCHAR2 (50);
      err_msg      VARCHAR2 (50);
      err_num      NUMBER;
      w_idpartit   INTEGER;
   BEGIN
     -- preparar les  dades per taula LOG
      p_nompro := 'Hi Ha Partit';
      p_inlog := p_idpartit;
      w_actiu := 'N';
  -- consulta a la taula partit si la clau existeix 
      SELECT idpartit
      INTO   w_idpartit
      FROM   PARTIT
      WHERE  (idpartit = p_idpartit);

      p_outlog := p_idpartit;
      RSP := 'OK';
      w_actiu := 'S';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,RSP);
      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: NO Hi Ha Partit';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
          RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
          RETURN w_actiu;
   END hiHaPartit;

   /******************************************************************************
       NOM:         HIHACARREC
       PROPSIT:    Retorna l'estat d'un registre de la taula CARREC
       PARAMETERS:
       INPUT:       P_IDCARREC
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CLAU NO EXISTEIX.
                    'S'  SI LA CLAU EXISTEIX
       ******************************************************************************/

   FUNCTION hiHaCarrec (p_idcarrec IN INTEGER)
      RETURN VARCHAR2 IS
      RSP          VARCHAR2 (50);
      w_actiu      VARCHAR2 (1);
      p_nompro     VARCHAR2 (50);
      p_inlog      VARCHAR2 (500);
      p_outlog     VARCHAR2 (500);
      p_rsp        VARCHAR2 (350);
      err_msg      VARCHAR2 (50);
      err_num      NUMBER;
      w_idcarrec   INTEGER;
   BEGIN
     -- preparar les  dades per taula LOG
      p_nompro := 'Hi Ha Carrec';
      p_inlog := p_idcarrec;
      w_actiu := 'N';
  -- consulta a la taula carrec persona si la clau existeix 
      SELECT idcarrec
      INTO   w_idcarrec
      FROM   CARREC
      WHERE  (idcarrec = p_idcarrec);

      p_outlog := p_idcarrec;
      RSP := 'OK';
      w_actiu := 'S';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,RSP);
      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: NO Hi Ha Carrec';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
       RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,RSP);
         COMMIT;
         RETURN w_actiu;
   END hiHaCarrec;

   /******************************************************************************
       NOM:         HIHACATEGORIA
       PROPSIT:    Retorna el resultat de la codificaci del camp idcategoria de la taula DESPESA
       PARAMETERS:
       INPUT:       P_IDCATEGORIA
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CODIFICACIO S ERRONIA.
                    'S'  SI LA CODIFICACIO S CORRECTA
       ******************************************************************************/
   FUNCTION hiHaCategoria (p_idCategoria IN INTEGER)
      RETURN VARCHAR2 IS
      RSP             VARCHAR2 (50);
      w_actiu         VARCHAR2 (1);
      p_nompro        VARCHAR2 (50);
      p_inlog         VARCHAR2 (500);
      p_outlog        VARCHAR2 (500);
      p_rsp           VARCHAR2 (350);
      err_msg         VARCHAR2 (50);
      err_num         NUMBER;
      w_idCategoria   INTEGER;
      es_error        EXCEPTION;
   BEGIN
     -- preparar les  dades per taula LOG
      p_nompro := 'Hi Ha Categoria';
      p_inlog := p_idCategoria;
      w_actiu := 'N';

    -- COMPROVA SI LA CATEGORIA S '0' (o) '1'
      IF p_idcategoria < 0 OR p_idcategoria > 1 THEN
         RAISE es_error;
      END IF;

      w_actiu := 'S';
      p_outlog := p_idcategoria;
      RSP := 'OK';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);

      RETURN w_actiu;
   EXCEPTION
      WHEN es_error THEN
         rsp := 'ERROR: NO Hi Ha Categoria';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
       RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
     RETURN w_actiu;
   END hiHaCategoria;


       PARAMETERS:
   /******************************************************************************
       NOM:         HIHAPROVEIDOR
       PROPSIT:    Retorna l'estat d'un registre de la taula PROVEIDOR
       INPUT:       P_IDPROVEIDOR
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CLAU NO EXISTEIX.
                    'S'  SI LA CLAU EXISTEIX
       ******************************************************************************/
   FUNCTION hiHaproveidor (p_idproveidor IN INTEGER)
      RETURN VARCHAR2 IS
      RSP             VARCHAR2 (50);
      w_actiu         VARCHAR2 (1);
      p_nompro        VARCHAR2 (50);
      p_inlog         VARCHAR2 (500);
      p_outlog        VARCHAR2 (500);
      p_rsp           VARCHAR2 (350);
      err_msg         VARCHAR2 (50);
      err_num         NUMBER;
      w_idproveidor   INTEGER;
   BEGIN
     -- preparar les  dades per taula LOG
      p_nompro := 'Hi Ha  proveidor';
      p_inlog := p_idproveidor;
      w_actiu := 'N';
     
        -- consulta a la taula proveidor  si la clau existeix 
      SELECT idproveidor
      INTO   w_idproveidor
      FROM   PROVEIDOR
      WHERE  (idproveidor = p_idproveidor);

      p_outlog := w_idproveidor;
      RSP := 'OK';
      w_actiu := 'S';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,RSP);

      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: NO Hi Ha  proveidor';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
    RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
   RETURN w_actiu;
   END hiHaProveidor;

   /******************************************************************************
       NOM:         HIHATIPUSDESPESA
       PROPSIT:    Retorna l'estat d'un registre de la taula TIPUS_DESPESA
       PARAMETERS:
       INPUT:       P_IDTIPUSDESPESA
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CLAU NO EXISTEIX.
                    'S'  SI LA CLAU EXISTEIX
       ******************************************************************************/
   FUNCTION hiHaTipusDespesa (p_idtipus IN INTEGER)
      RETURN VARCHAR2 IS
      RSP         VARCHAR2 (50);
      w_actiu     VARCHAR2 (1);
      p_nompro    VARCHAR2 (50);
      p_inlog     VARCHAR2 (500);
      p_outlog    VARCHAR2 (500);
      p_rsp       VARCHAR2 (350);
      err_msg     VARCHAR2 (50);
      err_num     NUMBER;
      w_idtipus   INTEGER;
   BEGIN
     -- preparar les  dades per taula LOG
      p_nompro := 'Hi Ha tipus Despesa ';
      p_inlog := p_idtipus;
      w_actiu := 'N';
  -- consulta a la taula tipus_despesa si la clau existeix 
      SELECT idtipus
      INTO   w_idtipus
      FROM   TIPUS_DESPESA
      WHERE  (idtipus = p_idtipus);

      p_outlog := p_idtipus;
      RSP := 'OK';
      w_actiu := 'S';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);

      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: NO Hi Ha tipus Despesa ';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
 RETURN w_actiu;
   END hiHaTipusDespesa;

   /******************************************************************************
       NOM:         HIHAESTATDESPESA
       PROPSIT:    Retorna l'estat d'un registre de la taula ESTAT
       PARAMETERS:
       INPUT:       P_IDESTATDESPESA
       OUTPUT:      ACTIU
       RETORNA:     'N'  SI LA CLAU NO EXISTEIX.
                    'S'  SI LA CLAU EXISTEIX
      RETURN VARCHAR2 IS
       ******************************************************************************/

   FUNCTION hiHaEstatDespesa (p_idestat IN INTEGER)
      RSP         VARCHAR2 (50);
      w_actiu     VARCHAR2 (1);
      p_nompro    VARCHAR2 (50);
      p_inlog     VARCHAR2 (500);
      p_outlog    VARCHAR2 (500);
      p_rsp       VARCHAR2 (350);
      err_msg     VARCHAR2 (50);
      err_num     NUMBER;
      w_idestat   INTEGER;
   BEGIN
     -- preparar les  dades per taula LOG
      p_nompro := 'Hi Ha estat Despesa ';
      p_inlog := p_idestat;
      w_actiu := 'N';
  -- consulta a la taula estat si la clau existeix 
      SELECT idestat
      INTO   w_idestat
      FROM   ESTAT
      WHERE  (idestat = p_idestat);

      p_outlog := p_idestat;
      RSP := 'OK';
      w_actiu := 'S';
      PAQ_UTI.altaLog (p_nompro
                      ,p_inlog
                      ,p_outlog
                      ,rsp);

      RETURN w_actiu;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         rsp := 'ERROR: NO Hi Ha Estat Despesa ';
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
     RETURN w_actiu;
      WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
    RETURN w_actiu;
   END hiHaEstatDespesa;
 /******************************************************************************
       NOM:         ANYOK
       PROPSIT:    VALIDAR SI LES DADES DEL PARAMETRE D'ENTRADA CORRESPONEN AL FORMAT ANY 
       PARAMETERS:
       INPUT:       P_ANY
       OUTPUT:      ACTIU
       RETORNA:     'ERROR ANY '  SI LA CODIFICACIO S ERRONIA .
                    'OK'  SI LA CODIFICACIO S CORRECTA
       ******************************************************************************/
   FUNCTION AnyOK (p_any VARCHAR2)
      RETURN VARCHAR2 IS
      RSP         VARCHAR2 (50);
      w_actiu     VARCHAR2 (1);
      p_nompro    VARCHAR2 (50);
      p_inlog     VARCHAR2 (500);
      p_outlog    VARCHAR2 (500);
      p_rsp       VARCHAR2 (350);
      err_msg     VARCHAR2 (50);
      err_num     NUMBER;
      v_num       NUMBER;

   BEGIN
   
      -- preparar les  dades per taula LOG
      p_nompro := 'AnyOK';
      p_inlog := p_any;
      w_actiu := 'N';
   -- valida que les posicions siguin numeriques
      v_num := NVL (LENGTH (TRIM (TRANSLATE (p_any, '0123456789', ' '))), 0);

      IF v_num > 0 THEN
         RETURN 'ERROR ANY';
      END IF;

      IF LENGTH (p_any) <> 4 THEN
         RETURN 'ERROR ANY';
      END IF;

      IF p_any > TO_CHAR (SYSDATE, 'YYYY') THEN
         RETURN 'ERROR ANY';
      END IF;
-- Si ha superat totes les validacions l'any s correcte
      RETURN 'OK';
      
  EXCEPTION
       WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR Any: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
    RETURN w_actiu;
      
      
      
   END AnyOK;

/******************************************************************************
       NOM:         CALCULARESTADISTICA
       PROPSIT:    COMPROVAR SI S'HAN MODIFICAT ELS CAMPS QUE INTERVENEN EN EL CALCUL
                    DEL MDUL ESTADISTIC PER EVITAR RE-CALCULAR LES ESTADISTIQUES QUAN  
                    NO HI HA CANVIS.
       PARAMETERS:
       INPUT:       P_DATADESPESA, P_IMPORTBRUT, P_TX100IVA, P_IDDESPEA
       OUTPUT:      ACTIU
       RETORNA:     'S' S'HA DE RECALCULAR EL MODUL ESTADSTIC.
                    'N' NO S'HA DE RECALCULAR EL MODUL ESTADSTIC.
       ******************************************************************************/
   FUNCTION calcularEstadistica (p_datadespesa    DATE
                                ,p_importbrut     NUMBER
                                ,p_tx100iva       NUMBER
                                ,p_iddespesa      INTEGER)
      RETURN VARCHAR2 IS
      v_datadespesa   DATE;
      v_importbrut    NUMBER;
      v_tx100iva      NUMBER;
      RSP         VARCHAR2 (50);
      w_actiu     VARCHAR2 (1);
      p_nompro    VARCHAR2 (50);
      p_inlog     VARCHAR2 (500);
      p_outlog    VARCHAR2 (500);
      p_rsp       VARCHAR2 (350);
      err_msg     VARCHAR2 (50);
      err_num     NUMBER;
      v_num       NUMBER;
   BEGIN
      -- preparar les  dades per taula LOG
      p_nompro := 'calcularEstadistica';
      p_inlog := 
              p_datadespesa 
      || ', ' || p_importbrut
      || ', ' || p_tx100iva 
      || ', ' || p_iddespesa; 
       w_actiu := 'N';
   
   -- OBTENIR ELS VALOR QUE TE LA DESPESA ABANS DE LA MODIFICACI
      SELECT DATADESPESA, IMPORTBRUT, TX100IVA
      INTO   v_datadespesa, v_importbrut, v_tx100iva
      FROM   DESPESA
      WHERE  IDDESPESA = p_iddespesa;
   -- COMPROVAR SI L'ANY DE LA DATA DE DESPESA A CANVIAT  
      IF TO_CHAR (v_datadespesa, 'YYYY') <> TO_CHAR (p_datadespesa, 'YYYY') THEN
         RETURN 'S';
      END IF;
   -- COMPROVAR SI EL IMPOT BRUT DE LA DESPESA A CANVIAT  
      IF v_importbrut <> p_importbrut THEN
         RETURN 'S';
      END IF;
  -- COMPROVAR SI EL %IVA DE LA DESPESA A CANVIAT 
      IF v_tx100iva <> p_tx100iva THEN
         RETURN 'S';
      END IF;
  -- SI HA SUPERAT TOTES LES VALIDACIONS S QUE NO HI HA CANVIS.
      RETURN 'N';
      
    EXCEPTION
       WHEN OTHERS THEN
         w_actiu := NULL;
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 50);
         rsp := 'ERROR: [' || TO_CHAR (err_num) || '] ' || err_msg;
         DBMS_OUTPUT.PUT_LINE (rsp);
         PAQ_UTI.altaLog (p_nompro
                         ,p_inlog
                         ,p_outlog
                         ,rsp);
         COMMIT;
      RETURN 'S';  
   END calcularEstadistica;
END PAQ_UTI;
/
